/**
 * 选择部门tree配置多选择部门
 */
 import { SERVICE_PLATFORM_POWER_BASE } from '@/config/serviceSettings.js'

 const PinyinMatch = require('pinyin-match').default

 export const getOrganization = (_this, data = {}) => {
   return _this
     .axios({
       url: SERVICE_PLATFORM_POWER_BASE + '/organization/all',
       data,
     })
     .then((response) => {
       if (response.success) {
         let ROOT_DEPARTURE = response.data.find((item) => {
           return item.parentId === '0'
         })
         _this.$set(_this.options, 'DEPARTMENT', response.data)
         _this.$set(_this.options, 'ROOT_DEPARTURE', ROOT_DEPARTURE)

         console.log(ROOT_DEPARTURE)
       }
     })
 }
 function resultTreeNodes(data, map = {}) {
   if (!data) return []
   data.forEach((item) => {
     delete item.children
     item.scopedSlots = { icon: 'custom' }
     item.isLeaf=true
     map[item.id] = item
   })
   let value = []
   data.forEach((item) => {
     var p = map[item.parentId]
     if (p) {
       if (!p.children) {
         p.children = []
       }
       p.isLeaf=false
       p.children.push(item)
     } else {
       value.push(item)
     }
   })
   return value
 }

 export default (self, view = 1) => {
   let user = self.$store.getters?.userInfo?.userCategory ?? null;

   return [

     {
       name: 'departmentCode',
       label: '部门code',
       hidden: true,
       get value() {
         if (user&&user !== 'I') {
           return self?.options.ROOT_DEPARTURE?.id ?? undefined;
         }
       }
     },
     {
       name: 'departmentId',
       label: '部门id',
       hidden: true,
       get value() {
         if (user&&user !== 'I') {
           return self?.options.ROOT_DEPARTURE?.id ?? undefined;
         }
       }
     },

     {
       name: 'department',
       get hidden() {
         //I-内部员工(自有) C-客户 V-供应商 H-货主 S-服务商(承运商)
         if (user === 'I') {
           if (self.st === view) {
             return true
           }
           return false
         } else {
           return true
         }
       },
       get value() {
         if (user&&user !== 'I') {
           return self?.options.ROOT_DEPARTURE?.code ?? undefined;
         }
       },
       type: 'a-tree-select',
       label: '部门',
       required: true,
       multiple: true,
       maxTagCount: 1,
       itemStyle: { marginTop: '6px'},
       className:'tms-tree-select',
       dropdownClassName: 'p-tree',
       treeNodeFilterProp: 'title',
       treeDefaultExpandAll:true,
       replaceFields: { key: 'id', title: 'name', value: 'id' },
       showSearch: false,
       treeCheckable: true,
       treeCheckStrictly:true,
       get treeDefaultExpandedKeys() {
         if (self.options?.DEPARTMENT?.length > 0) {
           let item = self.options.DEPARTMENT.find((item) => item.parentId == '0')
           return item?[item.id]:[]
         }
       },
       get treeData() {
         if (self.options?.DEPARTMENT?.length > 0) {
           return resultTreeNodes(self.options.DEPARTMENT)
         }
       },
       suffixIcon: () => {
         return self._c('a-icon', { class: 'picon', props: { theme: 'filled', type: 'folder' } })
       },
       filterTreeNode(inputValue, treeNode) {
         return PinyinMatch.match(treeNode.data.props.title, inputValue).length > 0 && treeNode
       },
       on: {
         change(value, label, extra, form) {

           if (!value||value.length===0) {
             return form.setFieldsValue({ departmentId: '', departmentName: '', departmentCode: '' })
           }
           label=[]
           value = value.map(item => {
             label.push(item.label)
             return item.value
           })
           let departmentId = value.join(), departmentCode=[],departmentName=label.join()
             self.options.DEPARTMENT.forEach(item => {
             if (departmentId.includes(item.id)) {
               departmentCode.push(item.code)
             }
           })
           departmentCode=departmentCode.join()
           form.setFieldsValue({ departmentId,departmentCode, departmentName})
         },
       },
       placeholder: '请选择部门',
     },

     {
       name: 'departmentName',
       label: '部门',
       type: 'txt',
       itemStyle:'white-space: nowrap;overflow: hidden;text-overflow: ellipsis;display:block',
       tooltip:true,
       get hidden() {
         //I-内部员工(自有) C-客户 V-供应商 H-货主 S-服务商(承运商)
         if (user === 'I') {
           if (self.st === view) {
             return false
           }
           return true
         } else {
           return false
         }
       },
       readOnly: true,
       get value() {
         if (user&&user !== 'I') {
           return self?.options.ROOT_DEPARTURE?.name ?? undefined;
         }
       }
     },

   ]
 }
