import client from 'webpack-theme-color-replacer/client'
import generate from '@ant-design/colors/lib/generate'
import { message } from 'ant-design-vue'

export const themeColor = {
  getAntdSerials(color) {
    // 淡化（即less的tint）
    const lightens = new Array(9).fill().map((t, i) => {
      return client.varyColor.lighten(color, i / 10)
    })
    // colorPalette 变换得到颜色值
    const colorPalettes = generate(color)
    const rgb = client.varyColor.toNum3(color.replace('#', '')).join(',')
    return lightens.concat(colorPalettes).concat(rgb)
  },
  changeColor(newColor) {
    const options = {
      newColors: this.getAntdSerials(newColor), // new colors array, one-to-one corresponde with `matchColors`
      changeUrl(cssUrl) {
        return `/${cssUrl}` // while router is not `hash` mode, it needs absolute path
      }
    }
    //console.log(options)
    return client.changer.changeColor(options, Promise)
  }
}

export const updateTheme = newPrimaryColor => {
  if (process.env.NODE_ENV === 'production') {
  const hideMessage = message.loading('正在切换主题', 0)
  themeColor.changeColor(newPrimaryColor).then(r => {
    hideMessage()
  })
  }
}

export const updateColorWeak = colorWeak => {
  // document.body.className = colorWeak ? 'colorWeak' : '';
  const app = document.body.querySelector('#app')
  colorWeak ? app.classList.add('colorWeak') : app.classList.remove('colorWeak')
}

export const updateHeaderStyle = headerStyle => {
  let cls = 'plg-headerstyle-' + headerStyle
  var header = document.querySelector('.ant-pro-global-header')
  var title = document.querySelector('.ant-pro-sider-menu-logo')
  if (!header || !title) {
    return
  }
  if (header.className.indexOf('plg-headerstyle-blue') > -1) {
    header.classList.remove('plg-headerstyle-blue')
  }
  if (header.className.indexOf('plg-headerstyle-black') > -1) {
    header.classList.remove('plg-headerstyle-black')
  }
  if (header.className.indexOf('plg-headerstyle-green') > -1) {
    header.classList.remove('plg-headerstyle-green')
  }

  if (title.className.indexOf('plg-headerstyle-blue') > -1) {
    title.classList.remove('plg-headerstyle-blue')
  }
  if (title.className.indexOf('plg-headerstyle-black') > -1) {
    title.classList.remove('plg-headerstyle-black')
  }
  if (title.className.indexOf('plg-headerstyle-green') > -1) {
    title.classList.remove('plg-headerstyle-green')
  }

  header.classList.add(cls)
  title.classList.add(cls)
}
