import Vue from 'vue'
import store from '@/store'

/**
 * Action 权限指令
 * 指令用法：
 *  - 在需要控制 action 级别权限的组件上使用 v-action:[method] , 如下：
 *    <i-button v-action:add >添加用户</a-button>
 *    <a-button v-action:delete>删除用户</a-button>
 *    <a v-action:edit @click="edit(record)">修改</a>
 *
 *  - 当前用户没有权限时，组件上使用了该指令则会被隐藏
 *  - 当后台权限跟 pro 提供的模式不同时，只需要针对这里的权限过滤进行修改即可
 *
 *  @see https://github.com/vueComponent/ant-design-vue-pro/pull/53
 */
const action = Vue.directive('action', {
  inserted: function(el, binding, vnode) {
    // 原理：判断页面注入的指令参数是否在 服务器返回的 权限数组里，在就显示元素。不在就隐藏元素
    const actionName = binding.arg // 指令参数
    const { currentOperations, currentPageElements } = store.getters
    const operations = currentOperations.concat(currentPageElements) // 过滤后的当前路由拥有的操作权限、元素权限
    if (operations.includes(actionName)) {
      return
    } else {
      if (el?.parentNode.className== 'ant-space-item') {
        return el.parentNode.remove()
       }
      ;(el.parentNode && el.parentNode.removeChild(el)) || (el.style.display = 'none')
    }
  }
})

export default action
