import { cargo_ownerUrl, driver_popup_Url, vehicle_popup_Url, vehicle_type_popup_Url, carrier_popup_Url } from '../../api'
import departmenTree from './bms_departmenTree.js'
import { UPCLOUD_RDC_TMS_MAINDATA_SERVICE } from '@/config/serviceSettings.js'

export function divider(self, value) {
  return {
    label: '',
    span: 24,
    itemHeight: '60px',
    labelCol: { span: 0 },
    wrapperCol: { span: 24 },
    type: () => {
      return self._c('a-divider', { attrs: { orientation: 'left' } }, [self._v(value)])
    },
  }
}
//发货商
import consignee from './consignee.js'
import shipper from './shipper.js'

const basePortName = {
  driverName: '',
  driverId: '',
  driverCode: '',
  driverPhoneNumber: '',
  affiliationCode: '',
  affiliationName: '',
  affiliationId: '',
  vehicleName: '',
  vehicleId: '',
  vehicleCode: '',
  weightLoadingRate: '',
  carrierCode: '',
  carrierId: '',
  carrierName: '',
  vehicleTypeName: '',
  vehicleTypeId: '',
  vehicleTypeCode: '',
  // vehicleType: '',
}

export const mainGetFormList = (self, expData = []) => {
  console.log(self);
  let inputType = self.st == 1 ? 'txt' : 'input'
  let searchModalType = self.st == 1 ? 'txt' : 'search'
  let selectMode = self.st == 1 ? 'txt' : ''
  let numberType = self.st == 1 ? true : false
  let e = []
  if (expData.length > 0) {
    e = expData.map((item) => {
      return { ...item }
    })
    e.forEach((item) => {
      let type = item.type
      Object.defineProperty(item, 'type', {
        get() {
          return self.st === 1 ? 'txt' : type
        },
      })
    })
    e = [divider(self, '扩展信息'), ...e]
  }

  let parcelTable = []
  if (self.parcelTable) {
    parcelTable = [
      {
        span: 24,
        itemHeight: 'auto',
        wrapperCol: { span: 24 },
        scopedSlots: { customRender: 'table_2' },
      },
    ]
  }

  //【订单管理】修改订单界面，运单号、货主、订单号、WMS单号 不能修改，置灰显示
  return [
    divider(self, '基本信息'),
    { name: 'id', label: 'id', hidden: true, value: '' },
    { name: 'source', label: 'source', hidden: true, value: '' },
    {
      name: 'shipbillNo',
      label: '运单号',
      value: '运单号自动生成',
      disabled: true,
      type: inputType,
    },
    {
      name: 'cargoOwnerName',
      value: '',
      label: '所属货主',
      required: true,
      targetClick: true,
      get disabled() {
        return self.st === 3
      },
      type: searchModalType,
      toolBar: {
        labelCol: { span: 8 },
        wrapperCol: { span: 16 },
        items: [
          {
            type: 'input',
            name: 'cargoOwnerCode',
            label: '货主编码',
            value: '',
          },
          {
            type: 'input',
            name: 'cargoOwnerName',
            label: '货主名称',
            value: '',
          },
        ],
      },
      remoteUrl: {
        url: cargo_ownerUrl,
        method: 'post',
      },
      rowKey: 'cargoOwnerId',
      targetValueKey: 'cargoOwnerName',
      on: {
        change(event, value, dataArr, form) {
          let obj = { cargoOwnerId: '', cargoOwnerCode: '' }
          let data = dataArr[0]
          let beforeId = form.getFieldValue('cargoOwnerId')
          if (!value) {
            self.cargoOwnerId = ''
            self.details = []
            form.resetFields()
            return
          }

          if (beforeId !== data.cargoOwnerId) {
            form.resetFields()
            self.details = []
          }
          obj = { cargoOwnerId: data.cargoOwnerId, cargoOwnerCode: data.cargoOwnerCode }
          self.cargoOwnerId = data.cargoOwnerId

          form.setFieldsValue(obj)
        },
      },
      tableConfig: {
        columns: [
          { dataIndex: 'cargoOwnerId', title: 'ID', hidden: true },
          { dataIndex: 'cargoOwnerName', title: '货主名称' },
          { dataIndex: 'cargoOwnerCode', title: '货主编码' },
        ],
      },
    },
    {
      name: 'cargoOwnerId',
      label: '所属货主ID',
      hidden: true,
    },
    {
      name: 'cargoOwnerCode',
      label: '所属货主code',
      hidden: true,
    },
    {
      name: 'orderId',
      label: '订单ID',
      hidden: true,
    },
    {
      name: 'wmsOrderId',
      label: 'WMS单据ID',
      hidden: true,
    },
    {
      name: 'orderNo',
      label: '订单号',
      value: '',

      get disabled() {
        return self.st === 3
      },
      type: self.st == 1 ? 'txt' : 'input',
    },
    {
      name: 'wmsOrderNo',
      label: ' WMS单号',
      get disabled() {
        return self.st === 3
      },
      value: '',
      type: self.st == 1 ? 'txt' : 'input',
    },
    {
      name: 'orderType',
      label: '订单类型',
      allowClear: false,
      required: true,
      mode: selectMode,
      type: 'select',
      value: 1,
      get options() {
        //订单类型 数据字典ORDER_TYPE：1普通 2紧急 3加急
        return self.options.ORDER_TYPE
      },
    },
    {
      name: 'orderBusinessType',
      label: '业务类型',
      type: 'select',
      mode: selectMode,
      value: 1,
      allowClear: false,
      required: true,
      get options() {
        return self.options.ORDER_BUSINESS_TYPE
      },
    },

    {
      name: 'deliveryMethodType',
      label: '配送方式',
      type: 'select',
      mode: self.st == 1 ? 'txt' : null,
      value: 1,
      allowClear: false,
      required: true,
      get options() {
        // DELIVERY_METHOD_TYPE：1配送 2自提
        return self.options.DELIVERY_METHOD_TYPE
      },
    },
    {
      name: 'transportType',
      label: '运输模式',
      type: 'select',
      mode: self.st == 1 ? 'txt' : null,
      value: 1,
      allowClear: false,
      required: true,
      get options() {
        //运输类型(运输模式) 数据字典TRANSPORT_TYPE:1整车 2零担
        return self.options.TRANSPORT_TYPE
      },
    },
    {
      name: 'temperatureRequirement',
      label: '温度要求',
      type: 'select',
      mode: selectMode,
      value: 1,
      allowClear: false,
      required: true,
      get options() {
        //温度要求 数据字典TEMPERATURE_REQUIREMENT：1常温 2冷藏 3冷冻 4恒温
        return self.options.TEMPERATURE_REQUIREMENT
      },
    },
    ...departmenTree(self),

    {
      name: 'memo',
      label: '备注',
      type: self.st == 1 ? 'txt' : 'input',
      span: 12,
      labelCol: { span: 3 },
      wrapperCol: { span: 20 },
      value: '',
    },
    ...e,
    divider(self, '时间信息'),
    {
      name: 'orderDate',
      label: '下单时间',
      showTime:true,
      type: self.st == 1 ? 'txt' : 'date',
      value: self.moment().format('YYYY-MM-DD HH:mm:ss'),
      format: 'YYYY-MM-DD HH:mm:ss',
      valueFrom: 'YYYY-MM-DD HH:mm:ss',
      placeholder: '年/月/日',
    },
    {
      name: 'submitDate',
      label: '提交时间',
      showTime:true,
      type: self.st == 1 ? 'txt' : 'date',
      value: self.moment().format('YYYY-MM-DD HH:mm:ss'),
      format: 'YYYY-MM-DD HH:mm:ss',
      valueFrom: 'YYYY-MM-DD HH:mm:ss',
      placeholder: '年/月/日',
    },
    {
      name: 'auditDate',
      label: '审核时间',
      showTime:true,
      type: self.st == 1 ? 'txt' : 'date',
      value: self.moment().format('YYYY-MM-DD HH:mm:ss'),
      format: 'YYYY-MM-DD HH:mm:ss',
      valueFrom: 'YYYY-MM-DD HH:mm:ss',
      placeholder: '年/月/日',
      style: 'margin-right:10%',
    },
    {
      name: 'startDate',
      label: '启运时间',
      showTime:true,
      type: self.st == 1 ? 'txt' : 'date',
      value: '',
      format: 'YYYY-MM-DD HH:mm:ss',
      valueFrom: 'YYYY-MM-DD HH:mm:ss',
      placeholder: '年/月/日',
      on: {
        change(event, value, form) {
          let deliveryDate = form.getFieldValue('deliveryDate')
          let transportAging = ''
          if (value && deliveryDate) {
            transportAging = (new Date(deliveryDate) - new Date(value)) / (1000 * 60 * 60)
          }
          form.setFieldsValue({ transportAging })
        },
      },
    },
    {
      name: 'deliveryDate',
      label: '送达时间',
      showTime:true,
      type: self.st == 1 ? 'txt' : 'date',
      value: '',
      format: 'YYYY-MM-DD HH:mm:ss',
      valueFrom: 'YYYY-MM-DD HH:mm:ss',
      placeholder: '年/月/日',
      style: 'margin-right:40%',
      on: {
        change(event, value, form) {
          let startDate = form.getFieldValue('startDate')
          let transportAging = ''
          if (startDate && value) {
            transportAging = (new Date(value) - new Date(startDate)) / (1000 * 60 * 60)
          }
          form.setFieldsValue({ transportAging })
        },
      },
    },
    {
      name: 'receiptDate',
      label: '回单时间',
      showTime:true,
      type: self.st == 1 ? 'txt' : 'date',
      value: '',
      format: 'YYYY-MM-DD HH:mm:ss',
      valueFrom: 'YYYY-MM-DD HH:mm:ss',
      placeholder: '年/月/日',
    },
    {
      name: 'originalBackDate',
      label: '正本回单时间',
      showTime:true,
      type: self.st == 1 ? 'txt' : 'date',
      value: '',
      format: 'YYYY-MM-DD HH:mm:ss',
      valueFrom: 'YYYY-MM-DD HH:mm:ss',
      placeholder: '年/月/日',
      style: 'margin-right:40%',
    },
    {
      name: 'transportAging',
      label: '时效',
      type: self.st == 1 ? 'txt' : 'number',
      value: '',
      unit: '小时',
      disabled: true,
    },
    ...shipper(self), //发货方
    ...consignee(self), //收货方
    {
      span: 24,
      itemHeight: 'auto',
      wrapperCol: { span: 24 },
      scopedSlots: { customRender: 'table' },
    },
    divider(self, '承运信息'),
    {
      name: 'carrierType',
      label: '承运方类型',
      allowClear: false,
      style: 'margin-right:75%',
      type: 'select',
      get value() {
        return self.carrierType
      },
      options: [
        { label: '承运商', value: 1 },
        { label: '司机', value: 2 },
      ],
      // get options() {
      //   return self.options.LINE_CARRIER_TYPE
      // },
      on: {
        change(value, label, options, form) {
          let obj = basePortName
          self.carrierType = value
          form.setFieldsValue(obj)
        },
      },
    },
    {
      type: 'searchModal',
      label: '司机',
      name: 'driverName',
      on: {
        change(event, value, dataArr, form) {
          let data = dataArr[0]
          let obj = {}
          if (data) {
            const { affiliationName, affiliationCode, affiliationId, phoneNumber, name, code, id, vehiclePlateNumber, vehicleId, vehicleCode, vehicleTypeName, vehicleTypeId, vehicleTypeCode } = data
            obj = {
              driverName: name,
              driverId: id,
              driverCode: code,
              driverPhoneNumber: phoneNumber,
              vehicleName: vehiclePlateNumber,
              vehicleId,
              vehicleCode,
              vehicleTypeName,
              vehicleTypeId,
              vehicleTypeCode,
              affiliationName,
              affiliationCode,
              affiliationId,
            }
          } else {
            obj = obj = {
              driverName: '',
              driverId: '',
              driverCode: '',
              driverPhoneNumber: '',
              vehicleName: '',
              vehicleId: '',
              vehicleCode: '',
              vehicleTypeName: '',
              vehicleTypeId: '',
              vehicleTypeCode: '',
              affiliationName: '',
              affiliationCode: '',
              affiliationId: '',
            }
          }
          form.setFieldsValue(obj)
        },
      },
      targetClick: true,
      get hidden() {
        return self.carrierType !== 2
      },
      placeholder: '请选择司机',
      targetValueKey: 'name',
      remoteUrl: { url: driver_popup_Url, method: 'post' },
      rowKey: 'id',
      toolBar: {
        items: [
          { type: 'input', name: 'name', label: '司机姓名', value: '' },
          { type: 'input', name: 'phoneNumber', label: '手机', value: '' },
        ],
      },
      tableConfig: {
        scroll: { x: true },
        columns: [
          { dataIndex: 'name', title: '司机姓名' },
          { dataIndex: 'typeName', title: '类型' },
          { dataIndex: 'phoneNumber', title: '手机' },
          { dataIndex: 'departmentName', title: '部门' },
          { dataIndex: 'affiliationName', title: '隶属机构' },
          { ellipsis: true, dataIndex: 'memo', title: '备注' },
        ],
      },
    },
    {
      name: 'driverCode',
      label: '司机code',
      hidden: true,
    },
    {
      name: 'driverId',
      hidden: true,
      label: '司机id',
    },
    {
      name: 'driverPhoneNumber',
      label: '司机手机号',
      get hidden() {
        return self.carrierType !== 2
      },
    },
    {
      name: 'affiliationCode',
      label: '隶属机构编码',
      hidden: true,
    },
    {
      name: 'affiliationId',
      label: '隶属机构ID',
      hidden: true,
    },
    {
      get hidden() {
        return self.carrierType !== 2
      },
      name: 'affiliationName',
      label: '隶属机构',
      style: 'margin-right:25%',
    },

    {
      get hidden() {
        return self.carrierType !== 2
      },
      type: 'searchModal',
      label: '车辆',
      name: 'vehicleName',
      targetClick: true,

      placeholder: '请选择车辆',
      targetValueKey: 'plateNumber',
      remoteUrl: { url: vehicle_popup_Url, method: 'post' },
      rowKey: 'id',
      toolBar: {
        items: [
          { type: 'input', name: 'code', label: '车辆编码', value: '' },
          { type: 'input', name: 'plateNumber', label: '车牌号', value: '' },
        ],
      },
      on: {
        change(event, value, dataArr, form) {
          let data = dataArr[0]
          let obj = {}
          if (data) {
            const { plateNumber, code, id } = data
            obj = {
              vehicleId: id,
              vehicleCode: code,
            }
          } else {
            obj = obj = {
              vehicleId: '',
              vehicleCode: '',
            }
          }
          form.setFieldsValue(obj)
        },
      },
      tableConfig: {
        scroll: { x: true },
        columns: [
          { dataIndex: 'id', title: 'id', hidden: true },
          { dataIndex: 'code', title: '车辆编码' },
          { dataIndex: 'plateNumber', title: '车牌号' },
          { dataIndex: 'typeName', title: '归属' },
          { dataIndex: 'affiliationName', title: '隶属机构' },
        ],
      },
    },
    {
      hidden: true,
      name: 'vehicleId',
      label: '车辆ID',
    },
    {
      hidden: true,
      name: 'vehicleCode',
      label: '车辆编号',
    },
    {
      hidden: true,
      name: 'vehicleTypeCode',
      label: '车型code',
    },
    {
      hidden: true,
      name: 'vehicleTypeId',
      label: '车型Id',
    },
    {
      get hidden() {
        return self.carrierType !== 2
      },
      type: 'searchModal',
      label: '车型',
      name: 'vehicleTypeName',
      placeholder: '请选择车型',
      targetValueKey: 'name',
      targetClick: true,
      remoteUrl: { url: vehicle_type_popup_Url, method: 'post' },
      rowKey: 'id',
      // toolBar: false,

      tableConfig: {
        scroll: { x: true },
        columns: [
          { dataIndex: 'id', title: 'ID' },
          { dataIndex: 'name', title: '车型名称' },
          { dataIndex: 'code', title: '车型编码' },
        ],
      },
      on: {
        change(event, value, dataArr, form) {
          let data = dataArr[0]
          let obj = {}
          if (data) {
            const { code, id } = data
            obj = {
              vehicleTypeCode: code,
              vehicleTypeId: id,
            }
          } else {
            obj = obj = {
              vehicleTypeCode: '',
              vehicleTypeId: '',
            }
          }
          form.setFieldsValue(obj)
        },
      },
    },
    {
      get hidden() {
        return self.carrierType !== 2
      },
      type: 'input',
      label: '装载率',
      // readOnly: true,
      name: 'weightLoadingRate',
    },

    {
      itemHeight: '80px',
      get hidden() {
        return self.carrierType !== 1
      },
      on: {
        change(event, value, dataArr, form) {
          let data = dataArr[0]
          let obj = {}
          if (data) {
            const { name, id } = data
            obj = {
              carrierName: name,
              carrierId: id,
            }
          } else {
            obj = obj = {
              carrierName: '',
              carrierId: '',
            }
          }
          form.setFieldsValue(obj)
        },
      },
      label: '承运商编码',
      name: 'carrierCode',
      type: 'searchModal',
      placeholder: '请选择承运商',
      targetClick: true,
      targetValueKey: 'code',
      remoteUrl: { url: UPCLOUD_RDC_TMS_MAINDATA_SERVICE + '/carrier/popup', method: 'post' },
      rowKey: 'id',
      toolBar: {
        items: [
          { labelCol: { span: 7 }, type: 'input', name: 'carrierCode', label: '承运商编码', value: '' },
          { labelCol: { span: 7 }, type: 'input', name: 'carrierName', label: '承运商名称', value: '' },
        ],
      },
      tableConfig: {
        columns: [
          { ellipsis: true, dataIndex: 'code', title: '承运商编码' },
          { ellipsis: true, dataIndex: 'name', title: '承运商名称' },
        ],
      },
    },
    {
      get hidden() {
        return self.carrierType !== 1
      },
      name: 'carrierName',
      label: '承运商名称',
      readOnly: true,
    },
    {
      hidden: true,
      name: 'carrierId',
      label: '承运商ID',
    },
    // ...parcelTable
  ]
}
