import { cargo_ownerUrl, queryCarriersUrl } from '../../js/api'
import { UPCLOUD_RDC_TMS_MAINDATA_SERVICE } from '@/config/serviceSettings.js'
const baseUrl = UPCLOUD_RDC_TMS_MAINDATA_SERVICE
import departmenTree from '@/blocks/components/formConfig/departmenTree.js'


const remote = (self, type) => {
  return {
    mode: self.st === 3 ? 'txt' : '',
    allowClear: false,
    get options() {
      return self.options[type]
    }
  }
}
export const setDetailsForm = (self, st) => {
  let selectedRowKeys = []

  let fns = () => {
    return self.formExtendedField.map(item => {
      let _item = { ...item }
      _item.type = st === 3 ? 'txt' : item.type
      return _item
    })
  }

  let userCategory = self.$store.getters?.userInfo?.userCategory ?? null


  return [
    //设备编号
    {
      type: 'input',
      name: 'id',
      hidden: true,
      label: '车辆id',
      value: ''
    },
    {
      type: st >= 2 ? 'txt' : 'input',
      name: 'code',
      required: true,
      label: '车辆编码',
      value: ''
    },
    {
      type: st >= 2 ? 'txt' : 'input',
      required: true,
      name: 'plateNumber',
      label: '车牌号',
      value: '',
      readOnly: st === 2 ? true : false
    },
    {
      name: 'plateNumberColor',
      type: 'select',
      label: '车辆颜色',
      mode: st === 3 ? 'txt' : '',
      required: true,
      get options() {
        return self.options.PLATE_NUMBER_COLOR
      },
    },
    ...departmenTree(self,3),

    {
      name: 'type',
      label: '归属',
      required: true,
      value: userCategory !== 'I' ? 2 : 1,

      allowClear: false,
      mode: st === 3 ? 'txt' : '',
      type: 'select',
      get options() {
      if (userCategory !== 'I' && self.options.VEHICLE_TYPE) {
        //非自有
        let VEHICLE_TYPE = []
        self.options.VEHICLE_TYPE.forEach((item, index) => {
          if (index !== 0) {
            VEHICLE_TYPE.push(item)
          }
        })
        return VEHICLE_TYPE
      }
      return self.options.VEHICLE_TYPE
    },

      on: {
        change(value, label, options, form) {
          self.affiliationType = value
          form.setFieldsValue({
            affiliationName: '',
            affiliationType: value > 1 ? 2 : '',
            affiliationId: '', affiliationCode: ''
          })
        }
      }
    },

    { name: 'affiliationId', label: '隶属机构ID', hidden: true },
    { name: 'affiliationType', label: '隶属机构类型', hidden: true, value: userCategory !== 'I' ? 2 : ''},
    { name: 'affiliationCode', label: '隶属机构编码', hidden: true },
    {
      name: 'affiliationName',
      label: '隶属机构',
      get disabled() {
        return self.affiliationType===1
      },
      get required() {
        return self.affiliationType === 1||userCategory == 'I'?false:true
      },
      get type() {
        return  st === 3 ? 'txt' : 'searchModal'
      },
      targetClick: true,
      toolBar: {
        labelCol: { span: 8 },
        wrapperCol: { span: 16 },
         items: [
            {
              type: 'input',
              name: 'code',
              label: '承运商编码' ,
              value: ''
            },
            {
              type: 'input',
              name: 'name',
              label: '承运商名称',
              value: ''
            }
          ]

      },

      remoteUrl: {
        url:queryCarriersUrl,
        method: 'post'
      },
      rowKey:'id',
      targetValueKey:'name',

      on: {
        change(...args) {
          let data = args[2][0],
            form = args[3],
            obj = null
          if (self.affiliationType > 1) {
            obj = { affiliationType: data.type, affiliationId: data.id, affiliationCode: data.code }
          } else {
            obj = { affiliationName: data.cargoOwnerType, affiliationId: data.cargoOwnerId, affiliationCode: data.cargoOwnerCode }
          }
          form.setFieldsValue(obj)
          selectedRowKeys = args[2]
        }
      },
      tableConfig: {
        //selectedRowKeys:"10000002",
        columns:[
          { dataIndex: 'id', title: 'ID', hidden: true },
          { dataIndex: 'name', title: '承运商名称' },
          { dataIndex: 'code', title: '承运商编码' }
        ]
      }
    },

    {
      type: st === 3 ? 'txt' : 'searchModal',
      name: 'vehicleTypeCode',
      label: '车型编码',
      mode: st === 3 ? 'txt' : '',
      required: true,
      targetClick:true,
      value: '',
      rowKey: 'id',
      targetValueKey: 'code',
      remoteUrl: {
        url: baseUrl + '/vehicle_type/popup',
        method: 'post'
      },
      on: {
        change(...args) {
          let data = args[2][0]
          let form = args[3]
          let obj = null

          if (!args[1]) {
            obj = {
              vehicleTypeCategory:'',
              vehicleTypeId: '',
              vehicleTypeName: '',
              brandName: '',
              length: '',
              width: '',
              height: '',
              maxMileage: '',
              weight: '',
              volume: ''
            }
          } else {
            obj = {
              vehicleTypeCategoryName: data.typeName,
              vehicleTypeId: data.id,
              vehicleTypeName: data.name || null,
              brandName: data.brandName || null,
              length: data.length || null,
              width: data.width || null,
              height: data.height || null,
              maxMileage: data.maxMileage || null,
              weight: data.weight || null,
              volume: data.volume || null
            }
          }
          form.setFieldsValue(obj)
        }
      },
      tableConfig: {
        columns: [
          { dataIndex: 'id', title: 'ID' },
          { dataIndex: 'name', title: '车型名称' },
          { dataIndex: 'code', title: '车型编码' }
        ]
      }
    },
    {
      readonly: true,
      name: 'vehicleTypeName',
      label: '车型名称',
      readOnly: true,

      type: st === 3 ? 'txt' : 'input'
    },
    {
      type: 'input',
      name: 'vehicleTypeId',
      label: '车型ID',
      hidden: true,
      value: ''
    },

    {
      type: st === 3 ? 'txt' : 'input',
      name: 'brandName',
      label: '品牌',
      value: '',
      readOnly:true,

     // ...remote(self, 'VEHICLE_TYPE_BRAND')
    },
    {
      type: st === 3 ? 'txt' : 'input',
      name: 'vehicleTypeCategoryName',
      label: '类别',
      value: '',
      readOnly:true,
      //...remote(self, 'VEHICLE_TYPE_CATEGORY')
    },
    {
      name: 'length',
      label: '长(m)',
      readOnly:true,

      type: st === 3 ? 'txt' : 'input'
    },

    {
      type: st === 3 ? 'txt' : 'input',
      name: 'width',
      label: '宽(m)',
      readOnly:true,

    },
    {
      type: st === 3 ? 'txt' : 'input',
      name: 'height',
      label: '高(m)',
      readOnly:true,

    },
    {
      type: st === 3 ? 'txt' : 'input',
      name: 'maxMileage',
      label: '最大行驶里程(公里)',
      readOnly:true,

    },
    {
      type: st === 3 ? 'txt' : 'input',
      name: 'weight',
      label: '载重(kg)',
      readOnly:true,

    },
    {
      type: st === 3 ? 'txt' : 'input',
      name: 'volume',
      label: '载方(m³)',
      readOnly:true,
    },
    // {
    //   type: 'select',
    //   name: 'deviceCompany',
    //   label: '设备厂商',
    //   value: '',
    //   ...remote(self, 'DEVICE_COMPANY')
    // },

    // {
    //   type: 'input',
    //   name: 'deviceId',
    //   label: '设备ID',
    //   hidden: true,
    //   value: ''
    // },
    // {
    //   type: 'input',
    //   name: 'deviceCode',
    //   label: '设备Code',
    //   hidden: true,
    //   value: ''
    // },
    // {
    //   name: 'deviceName',
    //   label: '设备号',
    //   value: '',
    //   type: st === 3 ? 'txt' : 'searchModal',
    //   targetClick: true,
    //   toolBar: {
    //     labelCol: { span: 8 },
    //     wrapperCol: { span: 16 },
    //     items: [
    //       {
    //         type: 'input',
    //         name: 'code',
    //         label: '设备编码',
    //         value: ''
    //       },
    //       {
    //         type: 'input',
    //         name: 'name',
    //         label: '设备名称',
    //         value: ''
    //       }
    //     ]
    //   },
    //   remoteUrl: {
    //     url: baseUrl + '/device/pop',
    //     method: 'post'
    //   },
    //   rowKey: 'id',
    //   targetValueKey: 'name',
    //   on: {
    //     change(...args) {
    //       let data = args[2][0],
    //         form = args[3]
    //       let obj = null
    //       if (!args[1]) {
    //         obj = { deviceId: '', deviceCode: '' }
    //       } else {
    //         obj = { deviceId: data.id, deviceCode: data.code }
    //       }

    //       form.setFieldsValue(obj)
    //     }
    //   },
    //   tableConfig: {
    //     columns: [
    //       { dataIndex: 'id', title: 'ID', hidden: true },
    //       { dataIndex: 'name', title: '设备名称' },
    //       { dataIndex: 'code', title: '设备编码' }
    //     ]
    //   }
    // },

    {
      name: 'mileageManagement',
      type: 'select',
      label: '里程管理',
      allowClear: false,
      mode: st === 3 ? 'txt' : '',
      get options() {
        return self.options.MILEAGE_MANAGEMENT
      },
    },
    {
      name: 'memo',
      label: '备注',
      span: 12,
      style: 'padding-left: 1.2%',
      wrapperCol: { span: 20   },
      labelCol: { span: 4 },
      type: st === 3 ? 'txt' : 'input'
    },
    ...fns()
  ]
}
