// /**分页查询接口重试日志 */
import request from '@/utils/request';
import { UPCLOUD_RDC_TMS_MAINDATA_SERVICE } from '@/config/serviceSettings.js'
const serviceName = UPCLOUD_RDC_TMS_MAINDATA_SERVICE;


//分页查询接口重试日志 GET
export const mainTableInfoUrl = serviceName + '/station/page';
export const viewUrl = serviceName + '/station/view';
export const childrenTableInfoUrl = serviceName + '/station/starting_station_query';



//站点批量导入
export const mainImportUrl = serviceName + '/station/batch_import';
export const mainDownUrl = serviceName + '/station/provider_download_template';
//14890 站点发货方导入模板下载
export const childrenDownUrl = serviceName + '/station/shipper_download_template';

//站点发货方批量导入
export const childrenImportUrl = serviceName + '/station/batch_import_shipper';




function responseThen(response) {
  if (response.success) {
    this.visible = false;
    this.$message.success(response.message);
  } else {
    this.$exception(response);
  }
  setTimeout(() => {
    this.confirmLoading = false;
  }, 1000);
  return response;
}


//站点新增|修改
export const mainSaveData = (_this, data, st) => {
  _this.confirmLoading = true;

  return request({
    url: serviceName + (st===3 ? '/station/update' : '/station/add'),
    method: 'post',
    data
  })
    .then(responseThen.bind(_this))
    .catch((response) => {
      _this.confirmLoading = false;
    });
};
export const getView = (_this, data, st) => {
  _this.confirmLoading = true;

  return request({
    url: viewUrl,
    method: 'post',
    data
  })
    .then(responseThen.bind(_this))
    .catch((response) => {
      _this.confirmLoading = false;
    });
};

export const childrenSaveData = (_this, data, st) => {
  _this.confirmLoading = true;

  return request({
    url: serviceName + (st=== 3 ? '/station/modify_shipper' : '/station/add_shipper'),
    method: 'post',
    data
  })
    .then(responseThen.bind(_this))
    .catch((response) => {
      _this.confirmLoading = false;
    });
};

export const mainIsEnable = (context, ids, status) => {
  return new Promise((resolve) => {
    context.$confirm({
      title: '是否' + (status > 0 ? '启用' : '停用') + '?',
      confirmLoading: context.confirmLoading,
      onOk(clear) {
        context.confirmLoading = true;
        return request({
          url: serviceName + '/station/'+(status > 0 ?'active':'disable'),
          method: 'post',
          data:ids,
        }).then((response) => {
          if (response.success) {
            clear();
            context.$refs.mainTable.reload();
          } else {
            context.$exception(response);
          }
          context.confirmLoading = false;
          return resolve(response);
        });
      }
    });
  });
};
export const zoneGet = data => {
  return request({
    url: serviceName + '/zone/get_zones',
    method: 'post',
    data,
  })
}

// 行政区域
export const getByPid = data => {
  return request({
    url: serviceName + '/zone/get-by-pid',
    method: 'post',
    data,
  })
};
export const deleteMainItem = (context, id) => {
  let confirmLoading = false;
  return new Promise((resolve) => {
    context.$confirm({
      title: '删除之后,不能再启用,确认是否删除!',
      confirmLoading,
      onOk(clear) {
        context.confirmLoading = true;
        return request({
          url: serviceName + '/station/batch_delete',
          method: 'post',
          data: id
        })
          .then((response) => {
            if (response.success) {
              clear();
              context.$refs.mainTable.reload();
            } else {
              context.$exception(response);
            }
            confirmLoading = false;
            return resolve(response);
          })
          .catch((error) => {
            confirmLoading = false;
          });
      }
    });
  });
};




export const paperIsEnable = (context, shipperIds, status, providerId) => {
  return new Promise((resolve) => {
    context.$confirm({
      title: '是否' + (status > 0 ? '启用' : '停用') + '?',
      confirmLoading: context.confirmLoading,
      onOk(clear) {
        context.confirmLoading = true;
        return request({
          url: serviceName + '/station/whether_to_enable_shipper',
          method: 'post',
          data: {
            status,
            shipperIds
          }
        }).then((response) => {
          if (response.success) {
            clear();
            context.$refs.childrenTable.reload({ data: { providerId } });
          } else {
            context.$exception(response);
          }
          context.confirmLoading = false;
          return resolve(response);
        });
      }
    });
  });
};


