import { cargo_ownerUrl } from '../../api'
import departmenTree from '@/blocks/components/formConfig/departmenTree.js'

export function divider(self, value) {
  return {
    label: '',
    span: 24,
    itemHeight: '60px',
    labelCol: { span: 0 },
    wrapperCol: { span: 24 },
    type: () => {
      return self._c('a-divider', { attrs: { orientation: 'left' } }, [self._v(value)])
    },
  }
}
//发货商
import consignee from './consignee.js'
import shipper from './shipper.js'


function ruleTimes(self, type) {
  let form = self.$refs[self.formName].form
  let {
    estimatedPickupStartTime,
    estimatedPickupStartTime_time,
    estimatedPickupEndTime,
    estimatedPickupEndTime_time,
    estimatedDeliveryStartTime,
    estimatedDeliveryStartTime_time,
    estimatedDeliveryEndTime,
    estimatedDeliveryEndTime_time,
  } = form.getFieldsValue()

  let a1 = estimatedPickupStartTime ? new Date(estimatedPickupStartTime + (estimatedPickupStartTime_time ? ' ' + estimatedPickupStartTime_time : '')).valueOf() : 0
  let a2 = estimatedPickupEndTime ? new Date(estimatedPickupEndTime + (estimatedPickupEndTime_time ? ' ' + estimatedPickupEndTime_time : '')).valueOf() : 0
  let b1 = estimatedDeliveryStartTime ? new Date(estimatedDeliveryStartTime + (estimatedDeliveryStartTime_time ? ' ' + estimatedDeliveryStartTime_time : '')).valueOf() : 0
  let b2 = estimatedDeliveryEndTime ? new Date(estimatedDeliveryEndTime + (estimatedDeliveryEndTime_time ? ' ' + estimatedDeliveryEndTime_time : '')).valueOf() : 0
  let time
  let obj = {
    [type]: false,
  }
  switch (type) {
    case 'a1':
      time = a2 + b1 + b2
      if (time === 0) {
        self.isTimeError = false
      } else {
        time = a2 || b1 || b2
        self.isTimeError = time < a1 ? true : false
      }
      break
    case 'a2':
      time = a1 + b1 + b2
      if (time === 0) {
        self.isTimeError = false
      } else {
        a1 = a1 || a2
        b1 = b1 || b2 || a2
        self.isTimeError = a1 > a2 || a2 > b1 ? true : false
      }
      break
    case 'b1':
      time = a1 + a2 + b2
      if (time === 0) {
        self.isTimeError = false
      } else {
        a2 = a2 || a1 || b1
        b2 = b2 || b1
        self.isTimeError = a2 > b1 || b1 > b2 ? true : false
      }
      break
    case 'b2':
      time = a1 + a2 + b1
      if (time === 0) {
        self.isTimeError = false
      } else {
        time = b1 || a2 || a1
        self.isTimeError = b2 < time
      }
      break
    default:
      self.isTimeError = false
  }
  obj[type] = self.isTimeError
  return obj
}
export const mainGetFormList = (self, expData = []) => {
  let inputType = self.st == 1 ? 'txt' : 'input'
  let searchModalType = self.st == 1 ? 'txt' : 'search'
  let selectMode = self.st == 1 ? 'txt' : ''
  let numberType = self.st == 1 ? true : false
  let e = []
  if (expData.length > 0) {
    e = expData.map((item) => {
      return { ...item }
    })
    e.forEach((item) => {
      let type = item.type
      Object.defineProperty(item, 'type', {
        get() {
          return self.st === 1 ? 'txt' : type
        },
      })
    })
    e = [divider(self, '扩展信息'), ...e]

  }

  let parcelTable = []
  if (self.parcelTable) {
    parcelTable = [
      {
        span: 24,
        itemHeight: 'auto',
        wrapperCol: { span: 24 },
        scopedSlots: { customRender: 'table_2' },
      },
    ]
  }

  //【订单管理】修改订单界面，运单号、货主、订单号、WMS单号 不能修改，置灰显示
  return [
    divider(self, '基本信息'),
    { name: 'shipbillId', label: 'id', hidden: true, value: '' },
    { name: 'status', label: '状态', hidden: true, value: '' },
    {
      name: 'shipbillNo',
      label: '运单号',
      value: '运单号自动生成',
      disabled: true,
      type: inputType,
    },
    {
      name: 'cargoOwnerName',
      value: '',
      label: '所属货主',
      required: true,
      targetClick: true,
      get disabled() {
        return self.st === 3
      },
      type: searchModalType,
      toolBar: {
        labelCol: { span: 8 },
        wrapperCol: { span: 16 },
        items: [
          {
            type: 'input',
            name: 'cargoOwnerCode',
            label: '货主编码',
            value: '',
          },
          {
            type: 'input',
            name: 'cargoOwnerName',
            label: '货主名称',
            value: '',
          },
        ],
      },
      remoteUrl: {
        url: cargo_ownerUrl,
        method: 'post',
      },
      rowKey: 'cargoOwnerId',
      targetValueKey: 'cargoOwnerName',
      on: {
        change(event, value, dataArr, form) {
          let obj = { cargoOwnerId: '', cargoOwnerCode: '' }
          let data = dataArr[0]
          let beforeId = form.getFieldValue('cargoOwnerId')
          if (!value) {
            self.cargoOwnerId = ''
            self.details = []
            form.resetFields()
            return
          }

          if (beforeId !== data.cargoOwnerId) {
            form.resetFields()
            self.details = []
          }
          obj = { cargoOwnerId: data.cargoOwnerId, cargoOwnerCode: data.cargoOwnerCode }
          self.cargoOwnerId = data.cargoOwnerId

          form.setFieldsValue(obj)
        },
      },
      tableConfig: {
        columns: [
          { dataIndex: 'cargoOwnerId', title: 'ID', hidden: true },
          { dataIndex: 'cargoOwnerName', title: '货主名称' },
          { dataIndex: 'cargoOwnerCode', title: '货主编码' },
        ],
      },
    },
    ...departmenTree(self),
    {
      name: 'cargoOwnerId',
      label: '所属货主ID',
      hidden: true,
    },
    {
      name: 'cargoOwnerCode',
      label: '所属货主code',
      hidden: true,
    },
    {
      name: 'orderId',
      label: '订单ID',
      hidden: true,
    },
    {
      name: 'wmsOrderId',
      label: 'WMS单据ID',
      hidden: true,
    },
    {
      name: 'orderNo',
      label: '订单号',
      value: '',

      get disabled() {
        return self.st === 3
      },
      type: self.st == 1 ? 'txt' : 'input',
    },
    {
      name: 'wmsOrderNo',
      label: ' WMS单号',
      get disabled() {
        return self.st === 3
      },
      value: '',
      type: self.st == 1 ? 'txt' : 'input',
    },
    {
      name: 'orderType',
      label: '订单类型',
      allowClear: false,
      required: true,
      mode: selectMode,
      type: 'select',
      value: 1,
      get options() {
        //订单类型 数据字典ORDER_TYPE：1普通 2紧急 3加急
        return self.options.ORDER_TYPE
      },
    },
    {
      name: 'orderBusinessType',
      label: '业务类型',
      type: 'select',
      mode: selectMode,
      value: 1,
      allowClear: false,
      required: true,
      get options() {
        return self.options.ORDER_BUSINESS_TYPE
      },
    },

    {
      name: 'deliveryMethodType',
      label: '配送方式',
      type: 'select',
      mode: self.st == 1 ? 'txt' : null,
      value: 1,
      allowClear: false,
      required: true,
      get options() {
        // DELIVERY_METHOD_TYPE：1配送 2自提
        return self.options.DELIVERY_METHOD_TYPE
      },
    },
    {
      name: 'transportType',
      label: '运输模式',
      type: 'select',
      mode: self.st == 1 ? 'txt' : null,
      value: 1,
      allowClear: false,
      required: true,
      get options() {
        //运输类型(运输模式) 数据字典TRANSPORT_TYPE:1整车 2零担
        return self.options.TRANSPORT_TYPE
      },
    },
    {
      name: 'temperatureRequirement',
      label: '温度要求',
      type: 'select',
      mode: selectMode,
      value: 1,
      allowClear: false,
      required: true,
      get options() {
        //温度要求 数据字典TEMPERATURE_REQUIREMENT：1常温 2冷藏 3冷冻 4恒温
        return self.options.TEMPERATURE_REQUIREMENT
      },
    },

    {
      name: 'memo',
      label: '备注',
      type: self.st == 1 ? 'txt' : 'input',
      span: 12,
      labelCol: { span: 3 },
      wrapperCol: { span: 20 },
      value: '',
    },
    ...e,


    divider(self, '时间信息'),
    {
      type: 'row',
      label: '订单下单日期',
      span: 24,
      labelCol: { span: 2 },
      wrapperCol: { span: 22 },
      list: [
        {
          itemStyle: 'display: inline-block;margin-right:5px;min-width:128px',
          name: 'orderDate',
          type: self.st == 1 ? 'txt' : 'date',
          value: '',
          placeholder: '年/月/日',
        },
      ],
    },
    {
      type: 'row',
      label: '预计提货时间',
      span: 24,
      labelCol: { span: 2 },
      wrapperCol: { span: 22 },

      list: [
        {
          name: 'estimatedPickupStartTime', //a1
          type: self.st == 1 ? 'txt' : 'date',
          value: '',
          format: 'YYYY-MM-DD',
          valueFrom: 'YYYY-MM-DD',
          placeholder: '年/月/日',
          style: 'min-width:128px;display: inline-block;margin-right:5px',

          // rules: [
          //   {
          //     message: null,
          //     required: false,
          //     validator: (rule, value, callback, Object, props) => {
          //       let { a1 } = ruleTimes(self, 'a1')
          //       if (!a1) {
          //         return callback()
          //       } else {
          //         callback(true)
          //       }
          //     },
          //   },
          // ],
          disabledDate: (startValue, form) => {
            let endValue = form.getFieldValue('estimatedPickupEndTime')
            if (!startValue || !endValue) {
              return false
            }
            startValue = startValue.format('YYYY-MM-DD')
            return new Date(startValue).valueOf() > new Date(endValue).valueOf()
          },
          on: {
            change() {
              ruleTimes(self, 'a1')

             },
            openChange(open, form) {
              if (!open) {
                form.focusByName('estimatedPickupStartTime_time')
              }
            },
          },
        },
        {
          hidden: self.st == 1,
          style: 'display: inline-block;min-width:54px',
          name: 'estimatedPickupStartTime_time', //a1_t
          placeholder: '小时/分钟',
          className: 'time',
          type: self.st == 1 ? 'txt' : 'a-time-picker',
          format: 'HH:mm',
          valueFormat: 'HH:mm',
          on: {
            change() {
             ruleTimes(self, 'a1')

            }
          },
          // rules: [
          //   {
          //     required: false,
          //     message: null,
          //     validator: (rule, value, callback, Object, props) => {
          //       let { a1 } = ruleTimes(self, 'a1')

          //       if (!a1) {
          //         return callback()
          //       } else {
          //         callback(true)
          //       }
          //     },
          //   },
          // ],
        },
        {
          style: `display: inline-block;text-align:center; padding:0 10px`,
          type: () => {
            return self._c('span', null, [self._v('--至--')])
          },
        },
        {
          name: 'estimatedPickupEndTime', //a2
          type: self.st == 1 ? 'txt' : 'date',
          valueFrom: 'YYYY-MM-DD',
          style: 'display: inline-block;margin-right:5px;min-width:128px',
          value: '',
          format: 'YYYY-MM-DD',
          placeholder: '年/月/日',

          // rules: [
          //   {
          //     message: null,
          //     required: false,
          //     validator: (rule, value, callback, Object, props) => {
          //       let { a2 } = ruleTimes(self, 'a2')
          //       if (!a2) {
          //         return callback()
          //       } else {
          //         callback(true)
          //       }
          //     },
          //   },
          // ],
          disabledDate: (a2, form) => {
            let a1 = form.getFieldValue('estimatedPickupStartTime')
            let b1 = form.getFieldValue('estimatedDeliveryStartTime') || form.getFieldValue('estimatedDeliveryEndTime') //b2
            if (a1 && a2 && !b1) {
              a2 = a2.format('YYYY-MM-DD')
              return new Date(a1).valueOf() > new Date(a2).valueOf()
            } else if (a1 && !a2 && b1) {
              return new Date(a1).valueOf() > new Date(b1).valueOf()
            } else if (a1 && a2 && b1) {
              a2 = a2.format('YYYY-MM-DD')
              if (new Date(a1).valueOf() > new Date(a2).valueOf() || new Date(a2).valueOf() > new Date(b1).valueOf()) {
                return true
              }
            }
            return false
          },

          on: {
            change() {
              ruleTimes(self, 'a2')

             },
            openChange(open, form) {
              if (!open) {
                form.focusByName('estimatedPickupEndTime')
              }
            },
          },
        },
        {
          style: 'display: inline-block;min-width:54px',
          hidden: self.st == 1,
          name: 'estimatedPickupEndTime_time',
          placeholder: '小时/分钟',
          type: self.st == 1 ? 'txt' : 'p-time-picker',
          format: 'HH:mm',
          valueFormat: 'HH:mm',
          value: '',
          on: {
            change() {
             ruleTimes(self, 'a2')

            }
          },
          // rules: [
          //   {
          //     message: null,
          //     required: false,
          //     validator: (rule, value, callback, Object, props) => {
          //       let { a2 } = ruleTimes(self, 'a2')
          //       if (!a2) {
          //         return callback()
          //       } else {
          //         callback(true)
          //       }
          //     },
          //   },
          // ],

        },
      ],
    },

    {
      type: 'row',
      label: '预计送达时间',
      span: 24,
      labelCol: { span: 2 },
      wrapperCol: { span: 22 },
      list: [
        {
          name: 'estimatedDeliveryStartTime', //b1
          type: self.st == 1 ? 'txt' : 'date',
          value: '',
          format: 'YYYY-MM-DD',
          valueFrom: 'YYYY-MM-DD',
          placeholder: '年/月/日',
          style: 'display: inline-block;margin-right:5px;min-width:128px',

          // rules: [
          //   {
          //     required: false,
          //     message: null,
          //     validator: (rule, value, callback, Object, props) => {
          //       let { b1 } = ruleTimes(self, 'b1')
          //       if (!b1) {
          //         return callback()
          //       } else {
          //         callback(true)
          //       }
          //     },
          //   },
          // ],
          disabledDate: (b1, form) => {
            let a2 = form.getFieldValue('estimatedPickupEndTime') || form.getFieldValue('estimatedPickupStartTime') //a1
            let b2 = form.getFieldValue('estimatedDeliveryEndTime')
            if (a2 && b1 && !b2) {
              b1 = b1.format('YYYY-MM-DD')
              return new Date(a2).valueOf() > new Date(b1).valueOf()
            } else if (!a2 && b1 && b2) {
              b1 = b1.format('YYYY-MM-DD')
              return new Date(b1).valueOf() > new Date(b2).valueOf()
            } else if (a2 && b1 && b2) {
              b1 = b1.format('YYYY-MM-DD')
              if (new Date(a2).valueOf() > new Date(b1).valueOf() || new Date(b1).valueOf() > new Date(b2).valueOf()) {
                return true
              }
            }

            return false
          },
          on: {
            change() {
             ruleTimes(self, 'b1')

            }
          },
        },
        {
          hidden: self.st == 1,
          style: 'display: inline-block;min-width:54px',
          className: 'time',
          name: 'estimatedDeliveryStartTime_time',
          type: self.st == 1 ? 'txt' : 'a-time-picker',
          format: 'HH:mm',
          valueFormat: 'HH:mm',
          value: '',
          placeholder: '小时/分钟',
          on: {
            change() {
             ruleTimes(self, 'b1')

            }
          },
          // rules: [
          //   {
          //     message: null,
          //     required: false,
          //     validator: (rule, value, callback, Object, props) => {
          //       let { b1 } = ruleTimes(self, 'b1')
          //       if (!b1) {
          //         return callback()
          //       } else {
          //         callback(true)
          //       }
          //     },
          //   },
          // ],
        },
        {
          style: `display: inline-block;text-align:center; padding:0 10px`,
          type: () => {
            return self._c('span', null, [self._v('--至--')])
          },
        },
        {
          name: 'estimatedDeliveryEndTime', //b2
          type: self.st == 1 ? 'txt' : 'date',
          format: 'YYYY-MM-DD',
          valueFrom: 'YYYY-MM-DD',
          placeholder: '年/月/日',
          style: 'display: inline-block;margin-right:5px;min-width:128px',
          value: '',
          disabledDate: (endValue, form) => {
            let startValue = form.getFieldValue('estimatedDeliveryStartTime')
            if (!startValue || !endValue) {
              return false
            }
            endValue = endValue.format('YYYY-MM-DD')
            return new Date(startValue).valueOf() > new Date(endValue).valueOf()
          },
          // rules: [
          //   {
          //     message: null,
          //     required: false,
          //     validator: (rule, value, callback, Object, props) => {
          //       let { b2 } = ruleTimes(self, 'b2')
          //       if (!b2) {
          //         return callback()
          //       } else {
          //         callback(true)
          //       }
          //     },
          //   },
          // ],
          on: {
            change() {
             ruleTimes(self, 'b2')

            }
          },
        },
        {
          style: 'display: inline-block;min-width:54px',
          hidden: self.st == 1,
          className: 'time',
          name: 'estimatedDeliveryEndTime_time',
          placeholder: '小时/分钟',
          type: self.st == 1 ? 'txt' : 'a-time-picker',
          format: 'HH:mm',
          valueFormat: 'HH:mm',
          value: '',
          on: {
            change() {
             ruleTimes(self, 'b2')

            }
          },
          // rules: [
          //   {
          //     message: null,
          //     required: false,
          //     validator: (rule, value, callback, Object, props) => {
          //       let { b2 } = ruleTimes(self, 'b2')
          //       if (!b2) {
          //         return callback()
          //       } else {
          //         callback(true)
          //       }
          //     },
          //   },
          // ],
        },
      ],
    },

    {
      labelCol: { span: 2 },
      span: 24,
      wrapperCol: { span: 2 },
      unit: '小时',
      name: 'transportAging',
      readOnly: true,
      inputType: 'number',
      label: '时效',
      tooltip: false,
      type: inputType,
      value: 0,
    },

    {
      span: 24,
      itemHeight: 'auto',
      wrapperCol: { span: 24 },
      type:null,
      scopedSlots: { customRender: 'timeRule' },
    },
    ...shipper(self), //发货方
    ...consignee(self), //收货方
    {
      span: 24,
      itemHeight: 'auto',
      wrapperCol: { span: 24 },
      scopedSlots: { customRender: 'table' },
    },
    // ...parcelTable
  ]
}

