

export const setMainColumns = (self, expData = []) => {
  return [
    {
      dataIndex: 'id',
      title: 'ID',
      hidden: true,
      customRender(txt, record) {
        if (!record.id) {
          record.id = record.shipbillId
        }
      },
    },
    { dataIndex: 'shipbillId', title: '运单Id', hidden: true },
    {
      dataIndex: 'shipbillNo',
      title: '运单号',
      width: 160,
      sorter: (a, b) => {
        if (!b.shipbillNo || !b.shipbillNo) return
        return a.shipbillNo.replace(/[^0-9]/gi, '') - b.shipbillNo.replace(/[^0-9]/gi, '')
      },
      customRender: (txt, record) => {
        return self._c(
          'a',
          {
            attrs: {
              href: 'javascript:void(0)',
            },
            on: {
              click: (event) => {
                event.stopPropagation()
                return self.$emit('view', record)
              },
            },
          },
          [self._v(txt ? txt : '')]
        )
      },
    },
    {
      dataIndex: 'cargoOwnerName',
      title: '货主',
      sorter: (a, b) => {
        return a.cargoOwnerName != b.cargoOwnerName
      },
    },

    {
      dataIndex: 'orderNo',
      title: '订单号',
      sorter: (a, b) => {
        if (!a.orderNo || !b.orderNo) {
          return
        }
        return a.orderNo.replace(/[^0-9]/gi, '') - b.orderNo.replace(/[^0-9]/gi, '')
      },
    },
    {
      dataIndex: 'wmsOrderNo',
      title: 'WMS单号',
      sorter: (a, b) => {
        if (!a.wmsOrderNo || !a.wmsOrderNo) return
        return a.wmsOrderNo.replace(/[^0-9]/gi, '') - b.wmsOrderNo.replace(/[^0-9]/gi, '')
      },
    },

    {
      dataIndex: 'orderTypeName',
      title: '订单类型',
      sorter: (a, b) => {
        return a.orderType - b.orderType
      },
    },
    {
      dataIndex: 'orderBusinessTypeName',
      title: '业务类型',
      sorter: (a, b) => {
        return a.orderBusinessType - b.orderBusinessType
      },
    },
    {
      dataIndex: 'deliveryMethodTypeName',
      title: '配送方式',
      align: 'center',
      sorter: (a, b) => {
        return a.deliveryMethodType - b.deliveryMethodType
      },
    },
    {
      dataIndex: 'transportTypeName',
      title: '运输模式',
      sorter: (a, b) => {
        return a.transportType - b.transportType
      },
    },
    {
      dataIndex: 'temperatureRequirementName',
      title: '温度要求',
      sorter: (a, b) => {
        return a.temperatureRequirement - b.temperatureRequirement
      },
    },
    {
      dataIndex: 'lineName',
      title: '线路',
      sorter: (a, b) => {
        return a.temperatureRequirement != b.temperatureRequirement
      },
    },
    {
      dataIndex: 'memo',
      title: '备注',
      width: 120,
      sorter: (a, b) => {
        return a.temperatureRequirement != b.temperatureRequirement
      },
    },
    ...expData,
    {
      dataIndex: 'orderDate',
      title: '订单下单日期',
      width: 120,

      sorter: (a, b) => {
        return +new Date(a.orderDate) - +new Date(b.orderDate)
      },
    },
    {
      dataIndex: 'estimatedPickupTime',
      title: '预计提货时间',
      width: 120,
      customRender(text, data) {
        if (!text) {
          return data.estimatedPickupStartTime
        }
        return text
      },
      sorter: (a, b) => {
        return +new Date(a.estimatedPickupStartTime) - +new Date(b.estimatedPickupStartTime)
      },
    },
    {
      dataIndex: 'estimatedDeliveryStartTime',
      title: '预计送达时间',
      width: 120,
      sorter: (a, b) => {
        return +new Date(a.estimatedDeliveryStartTime) - +new Date(b.estimatedDeliveryStartTime)
      },
    },
    {
      dataIndex: 'transportAging',
      title: '时效',
      width: 80,
      align: 'center',
      sorter: (a, b) => {
        return a.transportAging - b.transportAging
      },
    },
    {
      dataIndex: 'shipperTypeName',
      title: '发货方类型',
      width: 80,
      sorter: (a, b) => {
        return ~~a.shipperType - ~~b.shipperType
      },
    },
    {
      dataIndex: 'shipperUnitName',
      title: '发货方单位',
      width: 80,
      sorter: (a, b) => {
        return ~~a.shipperUnit - ~~b.shipperUnit
      },
    },
    {
      dataIndex: 'shipperName',
      title: '发货方',
      width: 80,
      sorter: (a, b) => {
        return a.shipperName.length - b.shipperName.length
      },
    },
    {
      dataIndex: 'shipperProvinceCityDistrict',
      title: '发货方目的地',
      width: 80,
      sorter: (a, b) => {
        if (a.shipperProvinceCityDistrict && b.shipperProvinceCityDistrict) {
          return a.shipperProvinceCityDistrict.length - b.shipperProvinceCityDistrict.length
        }
      },
    },
    {
      dataIndex: 'shipperPhoneNumber',
      title: '发货方手机号',
      sorter: (a, b) => {
        if (a.shipperPhoneNumber && a.shipperPhoneNumber) {
          return a.shipperPhoneNumber != b.shipperPhoneNumber
        }
      },
    },
    {
      dataIndex: 'shipperLandlineNumber',
      title: '发货方座机号',
      sorter: (a, b) => {
        if (a.shipperLandlineNumber && a.shipperLandlineNumber) {
          return a.shipperLandlineNumber != b.shipperLandlineNumber
        }
      },
    },
    {
      dataIndex: 'shipperEmail',
      title: '发货方邮箱',
      sorter: (a, b) => {
        if (a.shipperEmail && a.shipperEmail) {
          return a.shipperEmail.length - b.shipperEmail.length
        }
      },
    },
    {
      dataIndex: 'shipperAddress',
      title: '发货方详细地址',
      sorter: (a, b) => {
        if (a.shipperAddress && a.shipperAddress) {
          return a.shipperAddress.length - b.shipperAddress.length
        }
      },
    },
    {
      dataIndex: 'shipperMemo',
      title: '发货方备注',
      sorter: (a, b) => {
        if (a.shipper && a.shipper) {
          return a.shipper.length - b.shipperAddress.length
        }
      },
    },
    {
      dataIndex: 'shipperPositioning',
      title: '发货方位置',
      sorter: (a, b) => {
        return a.shipperPositioning - b.shipperPositioning
      },
    },

    {
      dataIndex: 'consigneeTypeName',
      title: '收货方类型',
      width: 80,
      sorter: (a, b) => {
        return ~~a.consigneeType - ~~b.consigneeType
      },
    },
    {
      dataIndex: 'consigneeUnitName',
      title: '收货方单位',
      width: 80,
      sorter: (a, b) => {
        return ~~a.consigneeUnit - ~~b.consigneeUnit
      },
    },
    {
      dataIndex: 'consigneeName',
      title: '收货方',
      width: 80,
      sorter: (a, b) => {
        return a.consigneeName.length - b.consigneeName.length
      },
    },
    {
      dataIndex: 'consigneeProvinceCityDistrict',
      title: '收货方目的地',
      width: 80,
      sorter: (a, b) => {
        if (a.consigneeProvinceCityDistrict && b.consigneeProvinceCityDistrict) {
          return a.consigneeProvinceCityDistrict.length - b.consigneeProvinceCityDistrict.length
        }
      },
    },
    {
      dataIndex: 'consigneePhoneNumber',
      title: '收货方手机号',
      sorter: (a, b) => {
        if (a.consigneePhoneNumber && a.consigneePhoneNumber) {
          return a.consigneePhoneNumber != b.consigneePhoneNumber
        }
      },
    },
    {
      dataIndex: 'consigneeLandlineNumber',
      title: '收货方座机号',
      sorter: (a, b) => {
        if (a.consigneeLandlineNumber && a.consigneeLandlineNumber) {
          return a.consigneeLandlineNumber != b.consigneeLandlineNumber
        }
      },
    },
    {
      dataIndex: 'consigneeEmail',
      title: '收货方邮箱',
      sorter: (a, b) => {
        if (a.consigneeEmail && a.consigneeEmail) {
          return a.consigneeEmail.length - b.consigneeEmail.length
        }
      },
    },
    {
      dataIndex: 'consigneeAddress',
      title: '收货方详细地址',
      sorter: (a, b) => {
        if (a.consigneeAddress && a.consigneeAddress) {
          return a.consigneeAddress.length - b.consigneeAddress.length
        }
      },
    },
    {
      dataIndex: 'consigneeMemo',
      title: '收货方备注',
      sorter: (a, b) => {
        if (a.consignee && a.consignee) {
          return a.consignee.length - b.consigneeAddress.length
        }
      },
    },
    {
      dataIndex: 'consigneePositioning',
      title: '收货方位置',
      sorter: (a, b) => {
        return a.consigneePositioning - b.consigneePositioning
      },
    },
    {
      dataIndex: 'totalPcs',
      title: '总件数',
      width: 120,
      sorter: (a, b) => {
        return a.totalPcs - b.totalPcs
      },
    },
    {
      dataIndex: 'totalWeight',
      title: '总重量',
      width: 120,
      sorter: (a, b) => {
        return a.totalWeight - b.totalWeight
      },
    },
    {
      dataIndex: 'totalVolume',
      title: '总体积',
      width: 120,
      sorter: (a, b) => {
        return a.totalVolume - b.totalVolume
      },
    },

    {
      dataIndex: 'packageWholePiece',
      title: '整件数',
      width: 120,
      sorter: (a, b) => {
        return a.packageWholePiece - b.packageWholePiece
      },
    },
    {
      dataIndex: 'packageLclPiece',
      title: '拼箱数',
      width: 120,
      sorter: (a, b) => {
        return a.packageLclPiece - b.packageLclPiece
      },
    },

    {
      dataIndex: 'packageTotalWeight',
      title: '包裹总重量',
      width: 120,
      sorter: (a, b) => {
        return a.packageTotalWeight - b.packageTotalWeight
      },
    },
    {
      dataIndex: 'packageTotalVolume',
      title: '包裹总体积',
      width: 120,
      sorter: (a, b) => {
        return a.packageTotalVolume - b.packageTotalVolume
      },
    },
  ]
}
