package com.prolog.rdc.tms.rmi.hystrix;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import com.prolog.framework.common.message.RestMessage;
import com.prolog.framework.core.pojo.Page;
import com.prolog.product.core.tools.util.LogUtils;
import com.prolog.rdc.tms.model.dto.DeviceByOneDTO;
import com.prolog.rdc.tms.model.dto.ExpensePopDTO;
import com.prolog.rdc.tms.model.dto.IntfErrorLogDTO;
import com.prolog.rdc.tms.model.dto.LmsTmsZoneDTO;
import com.prolog.rdc.tms.model.dto.LmsTmsZoneMinDTO;
import com.prolog.rdc.tms.model.dto.SettlementTypeDTO;
import com.prolog.rdc.tms.model.dto.SySParameterDTO;
import com.prolog.rdc.tms.model.dto.TmsBaseParameterDTO;
import com.prolog.rdc.tms.model.dto.VehicleCheckDTO;
import com.prolog.rdc.tms.model.dto.basicConfigure.ShipbillFitBusinessFlowDTO;
import com.prolog.rdc.tms.model.dto.basicConfigure.ShipbillMatchCarrierDTO;
import com.prolog.rdc.tms.model.dto.basicConfigure.ShipbillMatchLineDTO;
import com.prolog.rdc.tms.model.dto.basicConfigure.SignconfigPopDTO;
import com.prolog.rdc.tms.model.dto.basicConfigure.StandardTemplateDetailsListDTO;
import com.prolog.rdc.tms.model.dto.basicConfigure.StrategyBusinessMatchingDTO;
import com.prolog.rdc.tms.model.dto.basicConfigure.StrategySplitMatchingDTO;
import com.prolog.rdc.tms.model.dto.basicConfigure.StrategyTransferMatchingDTO;
import com.prolog.rdc.tms.model.dto.basicConfigure.SysAlertConfigurationQueryDTO;
import com.prolog.rdc.tms.model.dto.basicConfigure.SysAlertUsersQueryDTO;
import com.prolog.rdc.tms.model.dto.basicData.ApprovalLogDTO;
import com.prolog.rdc.tms.model.dto.basicData.ApprovalNodeSelDTO;
import com.prolog.rdc.tms.model.dto.basicData.BaseFeeDTO;
import com.prolog.rdc.tms.model.dto.basicData.BranchByEncodingNameDTO;
import com.prolog.rdc.tms.model.dto.basicData.BranchByNameDTO;
import com.prolog.rdc.tms.model.dto.basicData.CargoOwnerByEncodingNameDTO;
import com.prolog.rdc.tms.model.dto.basicData.CargoOwnerCheckDTO;
import com.prolog.rdc.tms.model.dto.basicData.CargoOwnerPageDTO;
import com.prolog.rdc.tms.model.dto.basicData.CarrierCheckDTO;
import com.prolog.rdc.tms.model.dto.basicData.CarriersByEncodingNameDTO;
import com.prolog.rdc.tms.model.dto.basicData.CustomerByEncodingNameDTO;
import com.prolog.rdc.tms.model.dto.basicData.CustomerCarrierDTO;
import com.prolog.rdc.tms.model.dto.basicData.ImportMatchingDataDTO;
import com.prolog.rdc.tms.model.dto.basicData.ItemByEncodingNameDTO;
import com.prolog.rdc.tms.model.dto.basicData.ItemCategoryCheckDTO;
import com.prolog.rdc.tms.model.dto.basicData.ItemCheckDTO;
import com.prolog.rdc.tms.model.dto.basicData.ItemPagingQueryDTO;
import com.prolog.rdc.tms.model.dto.basicData.LineByCustomerQueryDTO;
import com.prolog.rdc.tms.model.dto.basicData.LineShipperConsigneeDTO;
import com.prolog.rdc.tms.model.dto.basicData.ProviderCheckDTO;
import com.prolog.rdc.tms.model.dto.basicData.ProviderShipperCheckDTO;
import com.prolog.rdc.tms.model.dto.basicData.StationDistanceDTO;
import com.prolog.rdc.tms.model.dto.basicData.StationViewDTO;
import com.prolog.rdc.tms.model.dto.basicData.TmsArriverDTO;
import com.prolog.rdc.tms.model.dto.basicData.TmsBaseBranchApTimeDTO;
import com.prolog.rdc.tms.model.dto.basicData.TmsBaseBranchDTO;
import com.prolog.rdc.tms.model.dto.basicData.TmsBaseCarportDTO;
import com.prolog.rdc.tms.model.dto.basicData.TmsBaseCustomerDTO;
import com.prolog.rdc.tms.model.dto.basicData.TmsBaseCustomerReceiverNewDTO;
import com.prolog.rdc.tms.model.dto.basicData.TmsBaseTranslineViewDTO;
import com.prolog.rdc.tms.model.dto.basicData.TmsCarrierDTO;
import com.prolog.rdc.tms.model.dto.basicData.TmsLineMapDTO;
import com.prolog.rdc.tms.model.dto.basicData.TmsShipperDTO;
import com.prolog.rdc.tms.model.dto.basicData.TmsTransferStationByViewDTO;
import com.prolog.rdc.tms.model.dto.fleetManagement.ContainerDTO;
import com.prolog.rdc.tms.model.dto.fleetManagement.DriverCheckDTO;
import com.prolog.rdc.tms.model.dto.fleetManagement.DriverPopupDTO;
import com.prolog.rdc.tms.model.dto.fleetManagement.VehicleAndDriverDTO;
import com.prolog.rdc.tms.model.dto.fleetManagement.VehiclePopupDTO;
import com.prolog.rdc.tms.model.dto.fleetManagement.VehicleTypeCheckDTO;
import com.prolog.rdc.tms.model.dto.intelligent.BranchDTO;
import com.prolog.rdc.tms.model.dto.intelligent.ConsigneeDTO;
import com.prolog.rdc.tms.model.dto.intelligent.CustomerDTO;
import com.prolog.rdc.tms.model.dto.intelligent.VehicleDTO;
import com.prolog.rdc.tms.model.dto.intelligent.VehicleTypeDTO;
import com.prolog.rdc.tms.model.dto.message.SendMessageDTO;
import com.prolog.rdc.tms.model.entity.LmsTmsZoneModel;
import com.prolog.rdc.tms.model.entity.SysParamConfigModel;
import com.prolog.rdc.tms.model.entity.SysRefdatadModel;
import com.prolog.rdc.tms.model.vo.ImportLogUpdateVO;
import com.prolog.rdc.tms.model.vo.IntfCargoOwnerSaveVO;
import com.prolog.rdc.tms.model.vo.IntfCustomerConsigneeSaveVO;
import com.prolog.rdc.tms.model.vo.IntfCustomerSaveVO;
import com.prolog.rdc.tms.model.vo.IntfItemSaveVO;
import com.prolog.rdc.tms.model.vo.IntfProviderSaveVO;
import com.prolog.rdc.tms.model.vo.IntfProviderShipperSaveVO;
import com.prolog.rdc.tms.model.vo.ItemCategorySaveVO;
import com.prolog.rdc.tms.model.vo.LmsTmsZoneQueryVO;
import com.prolog.rdc.tms.model.vo.LmsTmsZoneVO;
import com.prolog.rdc.tms.model.vo.SysRefdatadSearchVO;
import com.prolog.rdc.tms.model.vo.TmsBaseParameterConditionVO;
import com.prolog.rdc.tms.model.vo.UpdateLatitudeAndLongitudeVO;
import com.prolog.rdc.tms.model.vo.basicConfigure.AlertMessageAddVO;
import com.prolog.rdc.tms.model.vo.basicConfigure.AlertMessageIsExistVO;
import com.prolog.rdc.tms.model.vo.basicConfigure.ApprovalBusinessNodeVO;
import com.prolog.rdc.tms.model.vo.basicConfigure.ApprovalOpenVO;
import com.prolog.rdc.tms.model.vo.basicConfigure.BusinessFlowShipbillVO;
import com.prolog.rdc.tms.model.vo.basicConfigure.ImportLogAddVO;
import com.prolog.rdc.tms.model.vo.basicConfigure.ShipbillMatchCarrierVO;
import com.prolog.rdc.tms.model.vo.basicConfigure.ShipbillMatchLineVO;
import com.prolog.rdc.tms.model.vo.basicConfigure.SignconfigPopVO;
import com.prolog.rdc.tms.model.vo.basicConfigure.StrategyBusinessMatchingVO;
import com.prolog.rdc.tms.model.vo.basicConfigure.StrategySplitMatchingVO;
import com.prolog.rdc.tms.model.vo.basicConfigure.StrategyTransferMatchingVO;
import com.prolog.rdc.tms.model.vo.basicConfigure.SysAlertConfigurationQueryVO;
import com.prolog.rdc.tms.model.vo.basicConfigure.SysAlertUsersQueryVO;
import com.prolog.rdc.tms.model.vo.basicData.AppointTimeQueryVO;
import com.prolog.rdc.tms.model.vo.basicData.BranchByEncodingNameVO;
import com.prolog.rdc.tms.model.vo.basicData.BranchByNameVO;
import com.prolog.rdc.tms.model.vo.basicData.CargoOwnerByEncodingNameVO;
import com.prolog.rdc.tms.model.vo.basicData.CargoOwnerCheckVO;
import com.prolog.rdc.tms.model.vo.basicData.CargoOwnerPageVO;
import com.prolog.rdc.tms.model.vo.basicData.CarriersByEncodingNameVO;
import com.prolog.rdc.tms.model.vo.basicData.CustomerByEncodingNameVO;
import com.prolog.rdc.tms.model.vo.basicData.CustomerCarrierVO;
import com.prolog.rdc.tms.model.vo.basicData.ImportMatchingDataVO;
import com.prolog.rdc.tms.model.vo.basicData.ItemByEncodingNameVO;
import com.prolog.rdc.tms.model.vo.basicData.ItemCategoryCheckVO;
import com.prolog.rdc.tms.model.vo.basicData.ItemCheckVO;
import com.prolog.rdc.tms.model.vo.basicData.ItemPagingQueryVO;
import com.prolog.rdc.tms.model.vo.basicData.LineByCustomerQueryVO;
import com.prolog.rdc.tms.model.vo.basicData.ProviderCheckVO;
import com.prolog.rdc.tms.model.vo.basicData.ProviderShipperCheckVO;
import com.prolog.rdc.tms.model.vo.basicData.SettlementTypeVO;
import com.prolog.rdc.tms.model.vo.basicData.StationDistanceVO;
import com.prolog.rdc.tms.model.vo.basicData.StationQueryVO;
import com.prolog.rdc.tms.model.vo.basicData.SysRefdatadQueryVO;
import com.prolog.rdc.tms.model.vo.basicData.TmsArriverVO;
import com.prolog.rdc.tms.model.vo.basicData.TmsBaseBranchQueryVO;
import com.prolog.rdc.tms.model.vo.basicData.TmsBaseTranslineVO;
import com.prolog.rdc.tms.model.vo.basicData.TmsCarrierViewVO;
import com.prolog.rdc.tms.model.vo.basicData.TmsLineShipperQueryVO;
import com.prolog.rdc.tms.model.vo.basicData.TmsTransferStationVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.DriverCheckVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.DriverPopupVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.VehicleAndVehicleVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.VehicleCheckVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.VehicleGpsSaveVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.VehiclePopupVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.VehicleTypeCheckVO;
import com.prolog.rdc.tms.rmi.feign.MaindataFeign;

import feign.hystrix.FallbackFactory;

/**
 * 
 *
 * 基础数据服务调用
 *
 * @className :MaindataHystrix
 * @author :fanzw
 * @creationTime :2022/05/17
 * @description :
 *
 * @modifier :fanzw
 * @modifyTime :2022/05/17
 * @reviseRemarks:
 *
 * @version :1.0.0
 *
 */
@Component
public class MaindataHystrix implements FallbackFactory<MaindataFeign> {

	private final Logger logger = LoggerFactory.getLogger(MaindataHystrix.class);

	@Override
	public MaindataFeign create(Throwable cause) {
		if (null != cause && StringUtils.hasLength(cause.getMessage())) {
			LogUtils.error(logger, cause.getMessage(), cause);
		}

		return new MaindataFeign() {
			@Override
			public RestMessage<String> getCodeNumberByCode(String code, String codetype) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|号码管理器|getCodeNumberByCode】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<String>> getCodeNumberByCodes(String code, String codetype, Integer codenum) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|号码管理器|getCodeNumberByCodes】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<Map<String, Object>>> getDropDownById(String objectcode, String groupcode1, String groupcode2, String code) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|数据字典翻译|getDropDownById】请求服务失败！", null);
			}

			@Override
			public RestMessage<Map<String, Object>> ListDropDownExecute(List<Map<String, Object>> param) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|数据字典批量查询|ListDropDownExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<String> parseCustomerConsigneeAddress(Long id, BigDecimal longitude, BigDecimal latitude, String address) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|客户管理|parseCustomerConsigneeAddress】请求服务失败！", null);
			}

			@Override
			public RestMessage<String> parseBranchAddress(Long id, BigDecimal longitude, BigDecimal latitude, String address) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|仓库管理|parseBranchAddress】请求服务失败！", null);
			}

			@Override
			public RestMessage<String> parseProviderShipperAddress(Long id, BigDecimal longitude, BigDecimal latitude, String address) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|供应商管理|parseProviderShipperAddress】请求服务失败！", null);
			}

			@Override
			public RestMessage<DeviceByOneDTO> deviceGetOne(Long id) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|设备管理|getOne】请求服务失败！", null);

			}

			@Override
			public RestMessage<VehicleCheckDTO> getVehicle(VehicleCheckVO vehicleCheckVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|车辆管理|getVehicle】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<SignconfigPopDTO>> getSignConfig(SignconfigPopVO signconfigPopVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|标签配置|popExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<String> addImportLogExecute(ImportLogAddVO importLogAddVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|导入日志保存|addImportLogExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<String> updateImportLogData(ImportLogUpdateVO updateVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|导入日志更新|updateImportLogData】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<StandardTemplateDetailsListDTO>> listStandardTemplateDetailsExecute(String importObjNo) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|标准版模板详情查询|listStandardTemplateDetailsExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<CargoOwnerByEncodingNameDTO> getCargoOwnerByEncodingNameExecute(CargoOwnerByEncodingNameVO cargoOwnerByEncodingNameVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据货主编码名称查询货主资料|getCargoOwnerByEncodingNameExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<Boolean> openExecute(ApprovalOpenVO approvalOpenVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|是否开启审批流程|openExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<Boolean> nodeApprovalExecute(ApprovalBusinessNodeVO approvalBusinessNodeVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|审批流程节点审核|nodeApprovalExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<Object> approvalInvalidExecute(String approvalBusinessId) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|审批作废|approvalInvalidExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<Object> approvalRetractExecute(String approvalBusinessId) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|审批撤回|approvalRetractExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<Object> approvalSubmitExecute(String approvalBusinessId) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|审批提交|approvalSubmitExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<TmsBaseParameterDTO>> getBaseParameterByConditionNoExecute(TmsBaseParameterConditionVO tmsBaseParameterConditionVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|获取系统参数|getBaseParameterByConditionNoExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<SysParamConfigModel> getValue(String code) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|获取系统参数|getValue】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<SysRefdatadModel>> getSysRefdatadDatas(SysRefdatadQueryVO vo) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|通用字典翻译|getSysRefdatadDatas】请求服务失败！", null);
			}

			@Override
			public RestMessage<ShipbillFitBusinessFlowDTO> shipbillMatchFlowQuery(BusinessFlowShipbillVO businessFlowShipbillVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|运单匹配业务流程|shipbillMatchFlowQuery】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<TmsCarrierDTO>> tmsCarrierView(@RequestBody TmsCarrierViewVO tmsCarrierViewVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|线路-承运方查看|tmsCarrierView】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<ApprovalLogDTO>> getBusLog(String approvalBusinessId) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|获取日志信息|getBusLog】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<ApprovalNodeSelDTO>> getBusDetail(String approvalBusinessId) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|获取节点信息|getBusDetail】请求服务失败！", null);
			}

			@Override
			public RestMessage<String> getQueryConfiguration(@RequestParam String bizType) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|获取高级查询配置|getQueryConfiguration】请求服务失败！", null);
			}

			@Override
			public RestMessage<Page<TmsArriverDTO>> tmsArriverPage(@RequestBody TmsArriverVO vo) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|获取线路的经纬度查询|tmsArriverPage】请求服务失败！", null);
			}

			@Override
			public RestMessage<TmsBaseBranchDTO> viewBranch(@RequestParam(value = "id") Long id) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|仓库查看|viewBranch】请求服务失败！", null);
			}

			@Override
			public RestMessage<LineByCustomerQueryDTO> getLineByCustomerExecute(@RequestBody LineByCustomerQueryVO lineByCustomerQueryVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据客户查询线路|getLineByCustomerExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<TmsBaseCarportDTO>> queryCarport(Long BranchId, String userId) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|仓库车位列表|queryCarport】请求服务失败！", null);
			}

			/**
			 * 车位列表查询
			 *
			 */
			@Override
			public RestMessage<List<TmsBaseCarportDTO>> queryCarportList(@RequestParam(value = "branchid") Long branchid) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|仓库车位列表|queryCarportList】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<TmsBaseBranchApTimeDTO>> queryAppointTime(AppointTimeQueryVO vo) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|仓库可预约时间段|queryAppointTime】请求服务失败！", null);
			}

			/**
			 * 预警配置查询
			 *
			 */
			@Override
			public RestMessage<List<SysAlertConfigurationQueryDTO>> listAlertConfigurationExecute(@RequestBody SysAlertConfigurationQueryVO sysAlertConfigurationQueryVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|预警配置查询|listAlertConfigurationExecute】请求服务失败！", null);
			}

			/**
			 * 预警配置用户查询
			 *
			 */
			@Override
			public RestMessage<List<SysAlertUsersQueryDTO>> listAlertUsersExecute(@RequestBody SysAlertUsersQueryVO sysAlertUsersQueryVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|预警配置用户查询|listAlertUsersExecute】请求服务失败！", null);
			}

			/**
			 * 预警消息是否存在
			 *
			 */
			@Override
			public RestMessage<Boolean> alertMessageIsExistExecute(@RequestBody AlertMessageIsExistVO alertMessageIsExistVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|预警消息是否存在|alertMessageIsExistExecute】请求服务失败！", null);
			}

			/**
			 * 预警消息新增
			 *
			 */
			@Override
			public RestMessage<String> alertMessageAddExecute(@RequestBody List<AlertMessageAddVO> alertMessageAddVOs) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|预警消息新增|alertMessageAddExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<BranchByNameDTO> getBranchByNameExecute(BranchByNameVO branchByNameVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据仓库名称查询仓库资料|getBranchByNameExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<ItemByEncodingNameDTO> getItemByEncodingNameExecute(ItemByEncodingNameVO itemByEncodingNameVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据商品编码名称查询商品资料|getItemByEncodingNameExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<ItemByEncodingNameDTO> getItemByEncodingNameExecuteIntf(ItemByEncodingNameVO itemByEncodingNameVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据商品编码名称查询商品资料|getItemByEncodingNameExecuteIntf】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<StrategySplitMatchingDTO>> matchingExecute(StrategySplitMatchingVO strategySplitMatchingVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|获取拆分策略|matchingExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<TmsBaseCustomerDTO> viewCustomerExecute(Long id) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|获取客户|viewCustomerExecute】请求服务失败！", null);

			}

			@Override
			public RestMessage<ProviderCheckDTO> getProviderExecute(ProviderCheckVO providerCheckVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|获取供应商|getProviderExecute】请求服务失败！", null);
			}

			/**
			 * 根据运单适配匹配承运方策略
			 *
			 * @param matchCarrierVOs
			 * @return
			 */
			@Override
			public RestMessage<Map<String, ShipbillMatchCarrierDTO>> shipbillMatchCarrierExecute(@RequestBody List<ShipbillMatchCarrierVO> matchCarrierVOs) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据运单适配匹配承运方策略|shipbillMatchCarrierExecute】请求服务失败！", null);
			}

			/**
			 * 客户收货方匹配唯一承运方
			 *
			 * @param carrierVO
			 * @return
			 */
			@Override
			public RestMessage<CustomerCarrierDTO> customerCarrierExecute(@RequestBody CustomerCarrierVO carrierVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|客户收货方匹配唯一承运方|customerCarrierExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<ItemCheckDTO> getItem(ItemCheckVO itemCheckVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|查看商品|getItem】请求服务失败！", null);
			}

			@Override
			public RestMessage<TmsBaseBranchDTO> getOneBranch(Long branchId) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|查看仓库|getOneBranch】请求服务失败！", null);
			}

			@Override
			public RestMessage<VehicleAndDriverDTO> vehicleAutomaticExecute(VehicleAndVehicleVO vehicleAndVehicleVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|司机车辆查询或新增|vehicleAutomaticExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<BranchByEncodingNameDTO> getBranchByEncodingNameExecute(BranchByEncodingNameVO branchByEncodingNameVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据仓库编码名称查询仓库资料|getBranchByEncodingNameExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<BranchByEncodingNameDTO> getBranchByEncodingNameExecuteIntf(BranchByEncodingNameVO branchByEncodingNameVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据仓库编码名称查询仓库资料|getBranchByEncodingNameExecuteIntf】请求服务失败！", null);
			}

			@Override
			public RestMessage<CustomerByEncodingNameDTO> getCustomerByEncodingNameExecute(CustomerByEncodingNameVO customerByEncodingNameVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据客户编码名称查询客户资料|getCustomerByEncodingNameExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<DriverCheckDTO> getDriverExecute(DriverCheckVO driverCheckVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|查询司机资料|getDriverExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<SettlementTypeDTO>> getSettlementByCode(SettlementTypeVO settlementTypeVO) {
				return RestMessage.newInstance(false, "【getSettlementByCode】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<BaseFeeDTO>> feeCode(SettlementTypeVO settlementTypeVO) {
				return RestMessage.newInstance(false, "【feeCode】请求服务失败！", null);
			}

			@Override
			public RestMessage<Page<BaseFeeDTO>> findBaseFeePage(ExpensePopDTO dto) {
				return RestMessage.newInstance(false, "【findBaseFeePage】请求服务失败！", null);
			}

			@Override
			public RestMessage<SySParameterDTO> queryParameter() {
				return RestMessage.newInstance(false, "【queryParameter】请求服务失败！", null);
			}

			@Override
			public RestMessage<Object> saveVehicleGps(VehicleGpsSaveVO vo) {
				return RestMessage.newInstance(false, "【saveVehicleGps】请求服务失败！", null);
			}

			@Override
			public RestMessage<ShipbillMatchLineDTO> shipbillMatchLineExecute(ShipbillMatchLineVO matchLineVO) {
				return RestMessage.newInstance(false, "【shipbillMatchLineExecute】匹配线路策略请求服务失败！", null);
			}

			@Override
			public RestMessage<List<TmsShipperDTO>> tmsLineShipperQuery(TmsLineShipperQueryVO lineShipperQueryVO) {
				return RestMessage.newInstance(false, "【tmsLineShipperQuery】线路发货方请求服务失败！", null);
			}

			@Override
			public RestMessage<List<IntfErrorLogDTO>> saveCustomerInterfaceExecute(List<IntfCustomerSaveVO> intfCustomerSaveVOs) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|客户接口转TMS|saveCustomerInterfaceExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<IntfErrorLogDTO>> saveProviderInterfaceExecute(List<IntfProviderSaveVO> intfProviderSaveVOs) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|供应商接口转TMS|saveProviderInterfaceExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<IntfErrorLogDTO>> saveProviderShipperInterfaceExecute(List<IntfProviderShipperSaveVO> intfProviderShipperSaveVOs) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|供应商发货方接口转TMS|saveProviderShipperInterfaceExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<IntfErrorLogDTO>> saveItemInterfaceExecute(List<IntfItemSaveVO> intfItemSaveVOs) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|商品接口转TMS|saveItemInterfaceExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<IntfErrorLogDTO>> saveItemCategoryInterfaceExecute(List<ItemCategorySaveVO> itemCategorySaveVOs) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|商品品类接口转TMS|saveItemCategoryInterfaceExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<IntfErrorLogDTO>> saveCustomerConsigneeInterfaceExecute(List<IntfCustomerConsigneeSaveVO> vos) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|客户收货方接口转主数据|saveCustomerConsigneeInterfaceExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<TmsLineMapDTO>> lineSites(List<Long> lineIds) {
				return RestMessage.newInstance(false, "【lineSites】线路站点查询请求服务失败！", null);
			}

			@Override
			public RestMessage<ImportMatchingDataDTO> importMatchingBranchDataExecute(ImportMatchingDataVO importMatchingDataVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|运单导入时匹配仓资料|importMatchingBranchDataExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<ImportMatchingDataDTO> importMatchingCustomerDataExecute(ImportMatchingDataVO importMatchingDataVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|运单导入时匹配客户资料|importMatchingCustomerDataExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<ImportMatchingDataDTO> importMatchingProviderDataExecute(ImportMatchingDataVO importMatchingDataVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|运单导入时匹配供应商资料|importMatchingProviderDataExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<StrategyTransferMatchingDTO>> matchingTransferExecute(@RequestBody StrategyTransferMatchingVO transferMatchingVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|匹配中转策略|matchingTransferExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<StrategyBusinessMatchingDTO>> matchingBusinessExecute(@RequestBody StrategyBusinessMatchingVO businessMatchingVO) {
				return RestMessage.newInstance(false, "【matchingExecute】匹配业务规则策略请求服务失败！", null);
			}

			@Override
			public RestMessage<TmsTransferStationByViewDTO> view(TmsTransferStationVO vo) {
				return RestMessage.newInstance(false, "【view】中转站信息查询请求服务失败！", null);
			}

			@Override
			public RestMessage<Object> sendMessage(SendMessageDTO dto) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|发送消息|sendMessage】请求服务失败！", null);
			}

			@Override
			public RestMessage<Page<LmsTmsZoneModel>> getZoneByConditions(@RequestBody LmsTmsZoneVO lmstmszonevo) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据名称/编码查询行政区域编码|getZoneByConditions】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<IntfErrorLogDTO>> saveCargoOwnerInterfaceExecute(List<IntfCargoOwnerSaveVO> intfCargoOwnerSaveVOs) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|货主接口转TMS|saveCargoOwnerInterfaceExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<TmsBaseCustomerReceiverNewDTO> viewCustomerReceiver(Long id) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|收货方查看|viewCustomerReceiver】请求服务失败！", null);
			}

			@Override
			public RestMessage<ProviderShipperCheckDTO> getShipperExecute(ProviderShipperCheckVO providerShipperCheckVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|发货方查看|getShipperExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<LmsTmsZoneMinDTO>> getAllZoneMin() {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|所有行政区域|getAllZoneMin】请求服务失败！", null);
			}

			@Override
			public RestMessage<Page<CargoOwnerPageDTO>> pageCargoOwnerExecute(CargoOwnerPageVO vo) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|货主资料分页查询|pageCargoOwnerExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<Page<ItemPagingQueryDTO>> pageItemExecute(ItemPagingQueryVO vo) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|商品资料分页查询|pageItemExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<Page<TmsBaseBranchDTO>> queryBranch(TmsBaseBranchQueryVO vo) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|仓库分页查询|queryBranch】请求服务失败！", null);
			}

			@Override
			public RestMessage<Object> updateBranchLatitudeAndLongitude(UpdateLatitudeAndLongitudeVO vo) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|更新仓库经纬度|updateBranchLatitudeAndLongitude】请求服务失败！", null);
			}

			@Override
			public RestMessage<Object> updateConsigneeLatitudeAndLongitude(UpdateLatitudeAndLongitudeVO vo) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|更新收货方经纬度|updateConsigneeLatitudeAndLongitude】请求服务失败！", null);
			}

			@Override
			public RestMessage<Object> updateShipperLatitudeAndLongitude(UpdateLatitudeAndLongitudeVO vo) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|更新发货方经纬度|updateShipperLatitudeAndLongitude】请求服务失败！", null);
			}

			/**
			 * 车辆资料弹窗
			 * 
			 * @param vehiclePopupVO
			 * @return
			 */
			@Override
			public RestMessage<Page<VehiclePopupDTO>> popupVehicleExecute(@RequestBody VehiclePopupVO vehiclePopupVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|车辆资料查询|popupVehicleExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<LmsTmsZoneDTO>> getLmsTmsZoneByNameExecute(LmsTmsZoneQueryVO lmstmszonequeryvo) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|行政区编码获取|getLmsTmsZoneByNameExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<Map<String, ShipbillMatchLineDTO>> shipbillMatchLineBatchExecute(List<ShipbillMatchLineVO> matchLineVOs) {
				return RestMessage.newInstance(false, "【shipbillMatchLineBatchExecute】运单批量匹配线路策略请求服务失败！", null);
			}

			@Override
			public RestMessage<List<SysRefdatadModel>> getSysRefdatadByName(SysRefdatadSearchVO sysRefdatadSearchVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|获取数据字典|getSysRefdatadByName】请求服务失败！", null);
			}

			@Override
			public RestMessage<CustomerByEncodingNameDTO> getCustomerByNameExecute(CustomerByEncodingNameVO customerByEncodingNameVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据客户名称查询客户资料|getCustomerByNameExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<ProviderCheckDTO> getProviderByNameExecute(@RequestBody ProviderCheckVO providerCheckVO) {
				return RestMessage.newInstance(false, "【getProviderByNameExecute】供应商查询失败！", null);
			}

			@Override
			public RestMessage<CargoOwnerCheckDTO> getCargoOwnerByNameExecute(CargoOwnerCheckVO cargoOwnerCheckVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据货主名称获取货主资料|getCargoOwnerByNameExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<ItemCheckDTO> getItemByNameExecute(ItemCheckVO itemCheckVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据商品名称获取商品资料|getItemByNameExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<ItemCategoryCheckDTO> getItemCategoryByNameExecute(ItemCategoryCheckVO itemCategoryCheckVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据商品品类名称获取商品品类资料|getItemCategoryByNameExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<VehicleTypeCheckDTO> getVehicleTypeByNameExecute(VehicleTypeCheckVO vehicleTypeCheckVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据车型名称查看车型资料|getVehicleTypeByNameExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<LineShipperConsigneeDTO>> queryLineConsignee(List<Long> lineIds) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|线路-收货方关联表查询|queryLineConsignee】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<StandardTemplateDetailsListDTO>> templateDetail(String templateId) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据主键查询模板详情|queryByTemplateId】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<StationDistanceDTO>> queryStationDistanceExecute(List<StationDistanceVO> vos) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|站点距离查询|queryStationDistanceExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<CarrierCheckDTO> getCarrierExecute(Long id) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|承运商查看|getCarrierExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<ContainerDTO>> findContainerByIdList(List<Long> idList) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|容器集合查询|findContainerByIdList】请求服务失败！", null);
			}

			@Override
			public RestMessage<TmsBaseTranslineViewDTO> viewLineByName(TmsBaseTranslineVO vo) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|线路信息查询|viewLineByName】请求服务失败！", null);
			}

			@Override
			public RestMessage<StationViewDTO> viewStationByNameExecute(StationQueryVO vo) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|站点信息查询|viewStationByNameExecute】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<VehicleDTO>> findVehicleByPlateNumberList(List<String> plateNumberList) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|查询车辆信息|findVehicleByPlateNumberList】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<VehicleDTO>> findVehicleByIdList(List<Long> idList) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|查询车辆信息|findVehicleByIdList】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<VehicleTypeDTO>> findVehicleTypeByIdList(List<Long> idList) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|查询车型信息|findVehicleTypeByIdList】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<BranchDTO>> findBranchByIdList(List<Long> idList) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|查询仓库信息|findBranchByIdList】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<CustomerDTO>> findCustomerByIdList(List<Long> idList) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|查询客户信息|findCustomerByIdList】请求服务失败！", null);
			}

			@Override
			public RestMessage<List<ConsigneeDTO>> findConsigneeByIdList(List<Long> idList) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|查询收货方信息|findConsigneeByIdList】请求服务失败！", null);
			}

			@Override
			public RestMessage<DriverPopupDTO> getDriverForCarriersAutoDis(DriverPopupVO driverPopupVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|查询司机,车辆信息|getDriverForCarriersAutoDis】请求服务失败！", null);
			}

			@Override
			public RestMessage<CarriersByEncodingNameDTO> getCarriersByEncodingNameExecute(CarriersByEncodingNameVO carriersByEncodingNameVO) {
				return RestMessage.newInstance(false, "【upcloud-rdc-tms-maindata-service|根据承运商编码名称查询承运商|getCarriersByEncodingNameExecute】请求服务失败！", null);
			}
		};
	}
}