package com.prolog.rdc.tms.app.model.vo.business;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;


/**
 * 待提货-运单模式查询VO
 *
 * Company: 湖北普罗格科技股份有限公司
 *
 * @author zhangsj
 * @since 2022/6/6 10:45
 */
@Data
@ApiModel("待提货-运单模式查询VO")
public class WaitPickUpGoodsQueryByShipbillVO implements Serializable{

    private static final long serialVersionUID = -6861629549788765464L;

    @ApiModelProperty(value = "货主")
    private String cargoOwnerName;

    @ApiModelProperty(value = "订单号")
    private String orderNo;

    @ApiModelProperty(value = "运单号")
    private String shipbillNo;

    @ApiModelProperty(value = "业务类型")
    private Integer orderBusinessType;

    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @ApiModelProperty(value = "预计提货开始日期")
    private Date estimatedPickupStartTime;

    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @ApiModelProperty(value = "预计提货结束日期")
    private Date estimatedPickupEndTime;

    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @ApiModelProperty(value = "预计送达开始日期")
    private Date estimatedDeliveryStartTime;

    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @ApiModelProperty(value = "预计送达结束日期")
    private Date estimatedDeliveryEndTime;

    @ApiModelProperty(value = "司机名称")
    private String driverName;

    @ApiModelProperty(value = "车辆号")
    private String vehiclePlateNumber;

    @ApiModelProperty(value = "线路")
    private String lineName;

    @ApiModelProperty(value = "页码")
    private Integer pageNum;

    @ApiModelProperty(value = "每页条数")
    private Integer pageSize;

    @ApiModelProperty(value = "企业ID",hidden = true)
    private Long enterpriseId;

    @ApiModelProperty(value = "用户类型",hidden = true)
    private String userType;

    @ApiModelProperty(value = "是否是部门权限",hidden = true)
    private String isDept;

    @ApiModelProperty(value = "部门权限",hidden = true)
    private String[] depts;

    @ApiModelProperty(value = "货主权限",hidden = true)
    private Long[] consignors;

    @ApiModelProperty(value = "仓库权限",hidden = true)
    private Long[] wsarehouses;

    @ApiModelProperty(value = "供应商权限",hidden = true)
    private Long[] suppliers;

    @ApiModelProperty(value = "客户权限",hidden = true)
    private Long[] customers;

    @ApiModelProperty(value = "单位ID",hidden = true)
    private Long unitId;

    @ApiModelProperty(value = "承运商ID",hidden = true)
    private Long carrierId;

    @ApiModelProperty(value = "开始时间")
    private Date startTime;

    @ApiModelProperty(value = "结束时间")
    private Date endTime;

    //STD-DEVELOP20220823 lhb 自动签到 20220907 start
    @ApiModelProperty(value = "司机经度")
    private BigDecimal driverLongitude;

    @ApiModelProperty(value = "司机纬度")
    private BigDecimal driverLatitude;
    //STD-DEVELOP20220823 lhb 自动签到 20220907 end

    @ApiModelProperty(value = "商品编码")
    private String itemCode;

    @ApiModelProperty(value = "始发地")
    private String shipperAddress;

    @ApiModelProperty(value = "目的地")
    private String consigneeAddress;

    @ApiModelProperty(value = "发货方名称")
    private String shipperName;

    @ApiModelProperty(value = "收货方名称")
    private String consigneeName;

    @ApiModelProperty(value = "发货方")
    private String shipperUnitName;

    @ApiModelProperty(value = "客户")
    private String consigneeUnitName;
}
