package com.prolog.rdc.tms.model.dto.pay;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.prolog.rdc.tms.model.dto.common.BmsBaseDTO;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 单据 收入费用主表 Model
 *
 * @author lhb
 * @since 2023-04-10 18:57
 */
@Data
public class BmsBillPayFeeDTO extends BmsBaseDTO {

    @ApiModelProperty(value = "ID")
    private Long id;

    @ApiModelProperty(value = "单据编号 运单编号/派车单编号")
    private String billNo;

    @ApiModelProperty(value = "派车单/运单 主键ID")
    private Long billId;

    @ApiModelProperty(value = "单据类型  运单/派车单 10 运单, 20 派车单")
    private Integer billType;

    @ApiModelProperty(value = "支出费用类型   1 支出托运费用, 2 支出司机费用")
    private Integer payFeeType;

    @ApiModelProperty(value = "结算状态 CHECK_STATUS 0：未结算，1：对账提交，2：对账确认，3：结算单生成 4 结算单审核")
    private Integer checkStatus;
    private String checkStatusName;

    @ApiModelProperty(value = "支出总价 类型为支出的含税金额")
    private BigDecimal payAmount;

    @ApiModelProperty(value = "合同支出总价 类型为收入的来源是价格表生成 含税金额")
    private BigDecimal contractPayAmount;

    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @ApiModelProperty(value = "计费时间")
    private Date costDate;

    @ApiModelProperty(value = "计费合同")
    private String costContract;

    @ApiModelProperty(value = "计费状态 1：计费成功，2：计费失败" )
    private Integer costStatus;

    @ApiModelProperty(value = "计费失败原因")
    private String costFailReason;

    @ApiModelProperty(value = "结算公司")
    private String settlementCompany;

    @ApiModelProperty(value = "结算公司id")
    private String settlementCompanyId;

    @ApiModelProperty(value = "结算公司编码")
    private String settlementCompanyCode;

    @ApiModelProperty(value = "结算部门")
    private String settlementDepartment;

    @ApiModelProperty(value = "结算部门ID")
    private String settlementDepartmentId;

    @ApiModelProperty(value = "结算部门编码")
    private String settlementDepartmentCode;

    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @ApiModelProperty(value = "核算时间")
    private Date settlementDate;

    @ApiModelProperty(value = "核算状态 0:未核算，1：核算待提交，2：核算审批中，3:核算完成 ")
    private Integer settlementStatus;

    @ApiModelProperty(value = "异常状态 0:非异常 1:异常")
    private Integer abnormalStatus;

    @ApiModelProperty(value = "扩展字段1")
    private String attrStr1;

    @ApiModelProperty(value = "扩展字段2")
    private String attrStr2;

    @ApiModelProperty(value = "扩展字段3")
    private String attrStr3;

    @ApiModelProperty(value = "扩展字段4")
    private String attrStr4;

    @ApiModelProperty(value = "扩展字段5")
    private String attrStr5;

    @ApiModelProperty(value = "费用明细")
    private List<BmsBillPayFeeDetailDTO> detailDTOList;

}