package com.prolog.rdc.tms.controller.fleetManagement;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.prolog.framework.common.message.RestMessage;
import com.prolog.framework.core.pojo.Page;
import com.prolog.framework.log.aop.annotation.UserLog;
import com.prolog.rdc.tms.model.dto.fleetManagement.TVmAccidentregDTO;
import com.prolog.rdc.tms.model.dto.fleetManagement.TVmFuelRecordDTO;
import com.prolog.rdc.tms.model.dto.fleetManagement.TVmIllegalRecordDTO;
import com.prolog.rdc.tms.model.dto.fleetManagement.TVmInsuranceDTO;
import com.prolog.rdc.tms.model.dto.fleetManagement.TmsRepairRecordDDTO;
import com.prolog.rdc.tms.model.dto.fleetManagement.TmsRepairRecordHDTO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TVmAccidentregAuditQueryVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TVmAccidentregQueryVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TVmAccidentregRegisterQueryVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TVmAccidentregVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TVmFuelRecordAuditQueryVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TVmFuelRecordQueryVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TVmFuelRecordRegisterQueryVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TVmFuelRecordVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TVmIllegalRecordAuditQueryVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TVmIllegalRecordQueryVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TVmIllegalRecordRegisterQueryVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TVmIllegalRecordVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TVmInsuranceAuditQueryVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TVmInsuranceQueryVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TVmInsuranceRegisterQueryVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TVmInsuranceVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TmsRepairRecordDVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TmsRepairRecordHAuditQueryVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TmsRepairRecordHQueryVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TmsRepairRecordHRegisterQueryVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TmsRepairRecordHVO;
import com.prolog.rdc.tms.model.vo.fleetManagement.TmsRepairRecordSaveVO;
import com.prolog.rdc.tms.service.fleetManagement.VehicleManageService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 车辆违章管理表 Controller
 * <p>
 * Copyright: Copyright (C) 2020 prolog-int, Inc. All rights reserved.
 * <p>
 * Company: 湖北普罗格科技股份有限公司
 *
 *
 * @author xiaoyf
 * @since 2020-06-03 11:27
 */
@RestController
@Api(tags = "车辆管理前端控制器")
@RequestMapping("/vehicleManage")
public class VehicleManageController {
	@Autowired
	private VehicleManageService service;

	@UserLog("车辆维保查询")
	@ApiOperation(value = "车辆维保查询", notes = "车辆维保查询")
	@PostMapping("/queryRepairRecord")
	public RestMessage<Page<TmsRepairRecordHDTO>> listTmsRepairRecordHExecute(@RequestBody TmsRepairRecordHQueryVO tmsRepairRecordHQueryVO) {
		try {
			Page<TmsRepairRecordHDTO> list = service.listTmsRepairRecordHData(tmsRepairRecordHQueryVO);
			return RestMessage.newInstance(true, "车辆维保查询成功", list);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@UserLog("车辆维保审核查询")
	@ApiOperation(value = "车辆维保审核查询", notes = "车辆维保审核查询")
	@PostMapping("/queryRepairRecordAudit")
	public RestMessage<Page<TmsRepairRecordHDTO>> listTmsRepairRecordHAuditExecute(@RequestBody TmsRepairRecordHAuditQueryVO tmsRepairRecordHAuditQueryVO) {
		try {
			Page<TmsRepairRecordHDTO> list = service.listTmsRepairRecordHAuditData(tmsRepairRecordHAuditQueryVO);
			return RestMessage.newInstance(true, "车辆维保审核查询成功", list);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@UserLog("车辆维保登记查询")
	@PostMapping("/queryRepairRecordRegister")
	@ApiOperation(value = "车辆维保登记查询", notes = "车辆维保登记查询")
	public RestMessage<Page<TmsRepairRecordHDTO>> listTmsRepairRecordHRegisterExecute(@RequestBody TmsRepairRecordHRegisterQueryVO tmsRepairRecordHRegisterQueryVO) {
		try {
			Page<TmsRepairRecordHDTO> list = service.listTmsRepairRecordHRegisterData(tmsRepairRecordHRegisterQueryVO);
			return RestMessage.newInstance(true, "车辆维保登记查询成功", list);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/queryTmsRepairRecordD")
	@UserLog("车辆维保明细信息查询")
	@ApiOperation(value = "车辆维保明细信息查询", notes = "车辆维保明细信息查询-/queryTmsRepairRecordD")
	public RestMessage<List<TmsRepairRecordDDTO>> queryTmsRepairRecordD(@RequestBody TmsRepairRecordDVO vo) {
		try {
			List<TmsRepairRecordDDTO> list = service.queryTmsRepairRecordD(vo);
			return RestMessage.newInstance(true, "操作成功！", list);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tmsRepairRecordSave")
	@UserLog("车辆维保信息保存")
	@ApiOperation(value = "车辆维保信息保存", notes = "车辆维保信息保存-/tmsRepairRecordSave")
	public RestMessage<Object> tmsRepairRecordSave(@RequestBody TmsRepairRecordSaveVO vo) {
		try {
			service.tmsRepairRecordSave(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tmsRepairRecordUpdate")
	@UserLog("车辆维保信息修改")
	@ApiOperation(value = "车辆维保信息修改", notes = "车辆维保信息修改-/tmsRepairRecordUpdate")
	public RestMessage<Object> tmsRepairRecordUpdate(@RequestBody TmsRepairRecordSaveVO vo) {
		try {
			service.tmsRepairRecordUpdate(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tmsRepairRecordDelete")
	@UserLog("车辆维保信息删除")
	@ApiOperation(value = "车辆维保信息删除", notes = "车辆维保信息删除-/tmsRepairRecordDelete")
	public RestMessage<Object> tmsRepairRecordDelete(@RequestBody List<TmsRepairRecordHVO> vo) {
		try {
			service.tmsRepairRecordDelete(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tmsRepairRecordDDelete")
	@UserLog("车辆维保明细删除")
	@ApiOperation(value = "", notes = "")
	public RestMessage<Object> tmsRepairRecordDDelete(@RequestBody List<TmsRepairRecordDVO> vo) {
		try {
			service.tmsRepairRecordDDelete(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tmsRepairRecordAudit")
	@UserLog("车辆维保审核")
	@ApiOperation(value = "车辆维保审核", notes = "车辆维保审核-/tmsRepairRecordAudit")
	public RestMessage<Object> tmsRepairRecordAudit(@RequestBody TmsRepairRecordHVO vo) {
		try {
			service.tmsRepairRecordAudit(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tmsRepairRecordReject")
	@UserLog("车辆维保驳回")
	@ApiOperation(value = "车辆维保驳回", notes = "车辆维保驳回-/tmsRepairRecordReject")
	public RestMessage<Object> tmsRepairRecordReject(@RequestBody TmsRepairRecordHVO vo) {
		try {
			service.tmsRepairRecordReject(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@UserLog("车辆违章查询")
	@ApiOperation(value = "车辆违章查询", notes = "车辆违章查询")
	@PostMapping("/queryTVmIllegalRecord")
	public RestMessage<Page<TVmIllegalRecordDTO>> listTVmIllegalRecordExecute(@RequestBody TVmIllegalRecordQueryVO tVmIllegalRecordQueryVO) {
		try {
			Page<TVmIllegalRecordDTO> tVmIllegalRecordDTOs = service.listTVmIllegalRecordData(tVmIllegalRecordQueryVO);
			return RestMessage.newInstance(true, "车辆违章查询成功", tVmIllegalRecordDTOs);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@UserLog("车辆违章审核查询")
	@ApiOperation(value = "车辆违章审核查询", notes = "车辆违章审核查询")
	@PostMapping("/queryTVmIllegalRecordAudit")
	public RestMessage<Page<TVmIllegalRecordDTO>> listTVmIllegalRecordAuditExecute(@RequestBody TVmIllegalRecordAuditQueryVO tVmIllegalRecordAuditQueryVO) {
		try {
			Page<TVmIllegalRecordDTO> tVmIllegalRecordDTOs = service.listTVmIllegalRecordAuditData(tVmIllegalRecordAuditQueryVO);
			return RestMessage.newInstance(true, "车辆违章审核查询成功", tVmIllegalRecordDTOs);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@UserLog("车辆违章登记查询")
	@ApiOperation(value = "车辆违章登记查询", notes = "车辆违章登记查询")
	@PostMapping("/queryTVmIllegalRecordRegister")
	public RestMessage<Page<TVmIllegalRecordDTO>> listTVmIllegalRecordRegisterExecute(@RequestBody TVmIllegalRecordRegisterQueryVO tVmIllegalRecordRegisterQueryVO) {
		try {
			Page<TVmIllegalRecordDTO> tVmIllegalRecordDTOs = service.listTVmIllegalRecordRegisterData(tVmIllegalRecordRegisterQueryVO);
			return RestMessage.newInstance(true, "车辆违章登记查询成功", tVmIllegalRecordDTOs);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmIllegalRecordSave")
	@UserLog("车辆违章登记保存")
	@ApiOperation(value = "车辆违章登记保存", notes = "车辆违章登记保存-/tVmIllegalRecordSave")
	public RestMessage<Object> tVmIllegalRecordSave(@RequestBody TVmIllegalRecordVO vo) {
		try {
			service.tVmIllegalRecordSave(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmIllegalRecordUpdate")
	@UserLog("车辆违章登记修改")
	@ApiOperation(value = "车辆违章登记修改", notes = "车辆违章登记修改-/tVmIllegalRecordUpdate")
	public RestMessage<Object> tVmIllegalRecordUpdate(@RequestBody TVmIllegalRecordVO vo) {
		try {
			service.tVmIllegalRecordUpdate(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmIllegalRecordDelete")
	@UserLog("车辆违章登记删除")
	@ApiOperation(value = "车辆违章登记删除", notes = "车辆违章登记删除-/tVmIllegalRecordDelete")
	public RestMessage<Object> tVmIllegalRecordDelete(@RequestBody List<TVmIllegalRecordVO> vo) {
		try {
			service.tVmIllegalRecordDelete(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmIllegalRecordAudit")
	@UserLog("车辆违章登记审核")
	@ApiOperation(value = "车辆违章登记审核", notes = "车辆违章登记审核-/tVmIllegalRecordAudit")
	public RestMessage<Object> tVmIllegalRecordAudit(@RequestBody TVmIllegalRecordVO vo) {
		try {
			service.tVmIllegalRecordAudit(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmIllegalRecordReject")
	@UserLog("车辆违章登记驳回")
	@ApiOperation(value = "车辆违章登记驳回", notes = "车辆违章登记驳回-/tVmIllegalRecordReject")
	public RestMessage<Object> tVmIllegalRecordReject(@RequestBody TVmIllegalRecordVO vo) {
		try {
			service.tVmIllegalRecordReject(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@UserLog("车辆事故查询")
	@ApiOperation(value = "车辆事故查询", notes = "车辆事故查询")
	@PostMapping("/queryTVmAccidentreg")
	public RestMessage<Page<TVmAccidentregDTO>> listTVmAccidentregExecute(@RequestBody TVmAccidentregQueryVO tVmAccidentregQueryVO) {
		try {
			Page<TVmAccidentregDTO> tVmAccidentregDTOs = service.listTVmAccidentregData(tVmAccidentregQueryVO);
			return RestMessage.newInstance(true, "车辆事故查询成功", tVmAccidentregDTOs);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@UserLog("车辆事故审核查询")
	@ApiOperation(value = "车辆事故审核查询", notes = "车辆事故审核查询")
	@PostMapping("/queryTVmAccidentregAudit")
	public RestMessage<Page<TVmAccidentregDTO>> listTVmAccidentregAuditExecute(@RequestBody TVmAccidentregAuditQueryVO tVmAccidentregAuditQueryVO) {
		try {
			Page<TVmAccidentregDTO> tVmAccidentregDTOs = service.listTVmAccidentregAuditData(tVmAccidentregAuditQueryVO);
			return RestMessage.newInstance(true, "车辆事故审核查询成功", tVmAccidentregDTOs);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@UserLog("车辆事故登记查询")
	@ApiOperation(value = "车辆事故登记查询", notes = "车辆事故登记查询")
	@PostMapping("/queryTVmAccidentregRegister")
	public RestMessage<Page<TVmAccidentregDTO>> listTVmAccidentregRegisterExecute(@RequestBody TVmAccidentregRegisterQueryVO tVmAccidentregRegisterQueryVO) {
		try {
			Page<TVmAccidentregDTO> tVmAccidentregDTOs = service.listTVmAccidentregRegisterData(tVmAccidentregRegisterQueryVO);
			return RestMessage.newInstance(true, "车辆事故登记查询成功", tVmAccidentregDTOs);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmAccidentregSave")
	@UserLog("车辆事故登记")
	@ApiOperation(value = "车辆事故登记", notes = "车辆事故登记-/tVmAccidentregSave")
	public RestMessage<Object> tVmAccidentregSave(@RequestBody TVmAccidentregVO vo) {
		try {
			service.tVmAccidentregSave(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmAccidentregUpdate")
	@UserLog("车辆事故修改")
	@ApiOperation(value = "车辆事故修改", notes = "车辆事故修改-/tVmAccidentregUpdate")
	public RestMessage<Object> tVmAccidentregUpdate(@RequestBody TVmAccidentregVO vo) {
		try {
			service.tVmAccidentregUpdate(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmAccidentregDelete")
	@UserLog("车辆事故删除")
	@ApiOperation(value = "车辆事故删除", notes = "车辆事故删除-/tVmAccidentregDelete")
	public RestMessage<Object> tVmAccidentregDelete(@RequestBody List<TVmAccidentregVO> vo) {
		try {
			service.tVmAccidentregDelete(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmAccidentregAudit")
	@UserLog("车辆事故审核")
	@ApiOperation(value = "车辆事故审核", notes = "车辆事故审核-/tVmAccidentregAudit")
	public RestMessage<Object> tVmAccidentregAudit(@RequestBody TVmAccidentregVO vo) {
		try {
			service.tVmAccidentregAudit(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmAccidentregReject")
	@UserLog("车辆事故驳回")
	@ApiOperation(value = "车辆事故驳回", notes = "车辆事故驳回-/tVmAccidentregReject")
	public RestMessage<Object> tVmAccidentregReject(@RequestBody TVmAccidentregVO vo) {
		try {
			service.tVmAccidentregReject(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@UserLog("车辆保险查询")
	@ApiOperation(value = "车辆保险查询", notes = "车辆保险查询")
	@PostMapping("/queryTVmInsurance")
	public RestMessage<Page<TVmInsuranceDTO>> listTVmInsuranceExecute(@RequestBody TVmInsuranceQueryVO tVmInsuranceQueryVO) {
		try {
			Page<TVmInsuranceDTO> tVmInsuranceDTOs = service.listTVmInsuranceData(tVmInsuranceQueryVO);
			return RestMessage.newInstance(true, "车辆保险查询成功", tVmInsuranceDTOs);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@UserLog("车辆保险审核查询")
	@ApiOperation(value = "车辆保险审核查询", notes = "车辆保险审核查询")
	@PostMapping("/queryTVmInsuranceAudit")
	public RestMessage<Page<TVmInsuranceDTO>> listTVmInsuranceAuditExecute(@RequestBody TVmInsuranceAuditQueryVO tVmInsuranceAuditQueryVO) {
		try {
			Page<TVmInsuranceDTO> tVmInsuranceDTOs = service.listTVmInsuranceAuditData(tVmInsuranceAuditQueryVO);
			return RestMessage.newInstance(true, "车辆保险审核查询成功", tVmInsuranceDTOs);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@UserLog("车辆保险登记查询")
	@ApiOperation(value = "车辆保险登记查询", notes = "车辆保险登记查询")
	@PostMapping("/queryTVmInsuranceRegister")
	public RestMessage<Page<TVmInsuranceDTO>> listTVmInsuranceRegisterExecute(@RequestBody TVmInsuranceRegisterQueryVO tVmInsuranceRegisterQueryVO) {
		try {
			Page<TVmInsuranceDTO> tVmInsuranceDTOs = service.listTVmInsuranceRegisterData(tVmInsuranceRegisterQueryVO);
			return RestMessage.newInstance(true, "车辆保险登记查询成功", tVmInsuranceDTOs);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmInsuranceSave")
	@UserLog("车辆保险登记保存")
	@ApiOperation(value = "车辆保险登记保存", notes = "车辆保险登记保存-/tVmInsuranceSave")
	public RestMessage<Object> tVmInsuranceSave(@RequestBody TVmInsuranceVO vo) {
		try {
			service.tVmInsuranceSave(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmInsuranceUpdate")
	@UserLog("车辆保险登记修改")
	@ApiOperation(value = "车辆保险登记修改", notes = "车辆保险登记修改-/tVmInsuranceUpdate")
	public RestMessage<Object> tVmInsuranceUpdate(@RequestBody TVmInsuranceVO vo) {
		try {
			service.tVmInsuranceUpdate(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmInsuranceDelete")
	@UserLog("车辆保险登记删除")
	@ApiOperation(value = "车辆保险登记删除", notes = "车辆保险登记删除-/tVmInsuranceDelete")
	public RestMessage<Object> tVmInsuranceDelete(@RequestBody List<TVmInsuranceVO> vo) {
		try {
			service.tVmInsuranceDelete(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmInsuranceAudit")
	@UserLog("车辆保险审核")
	@ApiOperation(value = "车辆保险审核", notes = "车辆保险审核-/tVmInsuranceAudit")
	public RestMessage<Object> tVmInsuranceAudit(@RequestBody TVmInsuranceVO vo) {
		try {
			service.tVmInsuranceAudit(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmInsuranceReject")
	@UserLog("车辆保险驳回")
	@ApiOperation(value = "车辆保险驳回", notes = "车辆保险驳回-/tVmInsuranceReject")
	public RestMessage<Object> tVmInsuranceReject(@RequestBody TVmInsuranceVO vo) {
		try {
			service.tVmInsuranceReject(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@UserLog("车辆加油查询")
	@ApiOperation(value = "车辆加油查询", notes = "车辆加油查询")
	@PostMapping("/queryTVmFuelRecord")
	public RestMessage<Page<TVmFuelRecordDTO>> listTVmFuelRecordExecute(@RequestBody TVmFuelRecordQueryVO tVmFuelRecordQueryVO) {
		try {
			Page<TVmFuelRecordDTO> tVmFuelRecordDTOs = service.listTVmFuelRecordData(tVmFuelRecordQueryVO);
			return RestMessage.newInstance(true, "车辆加油查询成功", tVmFuelRecordDTOs);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@UserLog("车辆加油审核查询")
	@ApiOperation(value = "车辆加油审核查询", notes = "车辆加油审核查询")
	@PostMapping("/queryTVmFuelRecordAudit")
	public RestMessage<Page<TVmFuelRecordDTO>> listTVmFuelRecordAuditExecute(@RequestBody TVmFuelRecordAuditQueryVO tVmFuelRecordAuditQueryVO) {
		try {
			Page<TVmFuelRecordDTO> tVmFuelRecordDTOs = service.listTVmFuelRecordAuditData(tVmFuelRecordAuditQueryVO);
			return RestMessage.newInstance(true, "车辆加油审核查询成功", tVmFuelRecordDTOs);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@UserLog("车辆加油登记查询")
	@ApiOperation(value = "车辆加油登记查询", notes = "车辆加油登记查询")
	@PostMapping("/queryTVmFuelRecordRegister")
	public RestMessage<Page<TVmFuelRecordDTO>> listTVmFuelRecordRegisterExecute(@RequestBody TVmFuelRecordRegisterQueryVO tVmFuelRecordRegisterQueryVO) {
		try {
			Page<TVmFuelRecordDTO> tVmFuelRecordDTOs = service.listTVmFuelRecordRegisterData(tVmFuelRecordRegisterQueryVO);
			return RestMessage.newInstance(true, "车辆加油登记查询成功", tVmFuelRecordDTOs);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmFuelRecordSave")
	@UserLog("车辆加油登记")
	@ApiOperation(value = "车辆加油登记", notes = "车辆加油登记-/tVmFuelRecordSave")
	public RestMessage<Object> tVmFuelRecordSave(@RequestBody TVmFuelRecordVO vo) {
		try {
			service.tVmFuelRecordSave(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmFuelRecordUpdate")
	@UserLog("车辆加油修改")
	@ApiOperation(value = "车辆加油修改", notes = "车辆加油修改-/tVmFuelRecordUpdate")
	public RestMessage<Object> tVmFuelRecordUpdate(@RequestBody TVmFuelRecordVO vo) {
		try {
			service.tVmFuelRecordUpdate(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmFuelRecordDelete")
	@UserLog("车辆加油删除")
	@ApiOperation(value = "车辆加油删除", notes = "车辆加油删除-/tVmFuelRecordDelete")
	public RestMessage<Object> tVmFuelRecordDelete(@RequestBody List<TVmFuelRecordVO> vo) {
		try {
			service.tVmFuelRecordDelete(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmFuelRecordAudit")
	@UserLog("车辆加油审核")
	@ApiOperation(value = "车辆加油审核", notes = "车辆加油审核-/tVmFuelRecordAudit")
	public RestMessage<Object> tVmFuelRecordAudit(@RequestBody TVmFuelRecordVO vo) {
		try {
			service.tVmFuelRecordAudit(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}

	@PostMapping("/tVmFuelRecordReject")
	@UserLog("车辆加油驳回")
	@ApiOperation(value = "车辆加油驳回", notes = "车辆加油驳回-/tVmFuelRecordReject")
	public RestMessage<Object> tVmFuelRecordReject(@RequestBody TVmFuelRecordVO vo) {
		try {
			service.tVmFuelRecordReject(vo);
			return RestMessage.newInstance(true, "操作成功！", null);
		} catch (Exception e) {
			e.printStackTrace();
			return RestMessage.newInstance(false, e.getMessage(), null);
		}
	}
}
