package com.prolog.rdc.tms.model.vo.xlsimport;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

import org.apache.poi.ss.usermodel.IndexedColors;
import org.springframework.format.annotation.DateTimeFormat;

import com.alibaba.excel.annotation.ExcelProperty;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.prolog.product.core.easyexcel.annotation.ValidationPropertyExt;
import com.prolog.product.core.easyexcel.support.DropdownPropExtConverter;
import com.prolog.product.core.easyexcel.util.ValidationCellEnum;

import lombok.Data;

/**
 * Company: 湖北普罗格科技股份有限公司
 *
 * @author zhangsj
 * @since 2022/3/28 15:16
 */
@Data
public class CarriersImportStdVO implements Serializable {

	private static final long serialVersionUID = -5218745850931052460L;

	@ValidationPropertyExt(headFontColor = IndexedColors.RED)
	@ExcelProperty(value = "承运商编码(必填)")
	private String code;

	@ValidationPropertyExt(headFontColor = IndexedColors.RED)
	@ExcelProperty(value = "承运商名称(必填)")
	private String name;

	@ExcelProperty(value = "承运商简称")
	private String shortName;

	@ValidationPropertyExt(headFontColor = IndexedColors.RED)
	@ExcelProperty(value = "部门名称(必填)")
	private String departmentName;

	@ExcelProperty(value = "合作性质", converter = DropdownPropExtConverter.class)
	@ValidationPropertyExt(dropdownValues = { "1", "2" }, dropdownNames = { "合同", "临时" }, cellType = ValidationCellEnum.LIST)
	private String cooperativeNature;

	@ExcelProperty(value = "税务登记号")
	private String taxRegisterNumber;

	@ExcelProperty(value = "营业执照号")
	private String businessLicenseNumber;

	@ExcelProperty(value = "道路许可证")
	private String roadPermit;

	@ExcelProperty(value = "备注")
	private String memo;

	@ExcelProperty(value = "联系人")
	private String contact;

	@ExcelProperty(value = "手机号")
	private String phoneNumber;

	@ExcelProperty(value = "座机号")
	private String landlineNumber;

	@ExcelProperty(value = "邮箱")
	private String email;

	@ExcelProperty(value = "邮编")
	private String postCode;

	@ExcelProperty(value = "省名称")
	private String provinceName;

	@ExcelProperty(value = "市名称")
	private String cityName;

	@ExcelProperty(value = "区名称")
	private String districtName;

	@ExcelProperty(value = "详细地址")
	private String address;

	@ExcelProperty(value = "法人代表")
	private String legalman;

	@ExcelProperty(value = "发票抬头")
	private String invoice;

	@ExcelProperty(value = "税号")
	private String invoiceTaxNumber;

	@ExcelProperty(value = "注册资金")
	private String registeredCapital;

	@ExcelProperty(value = "开户银行")
	private Integer bank;

	@ExcelProperty(value = "银行账号")
	private String bankAccount;

	@DateTimeFormat(pattern = "yyyy-MM-dd")
	@JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
	@ExcelProperty(value = "成立时间")
	private Date established;

	@ExcelProperty(value = "注册地址")
	private String registeredAddress;

//	@ExcelProperty(value = "提货签到(是/否)", converter = DropdownPropExtConverter.class)
//	@ValidationPropertyExt(dropdownValues = { "0", "1" }, dropdownNames = { "否", "是" }, cellType = ValidationCellEnum.LIST)
//	private String pickUpSign;
//
//	@ExcelProperty(value = "自动提货签到(是/否)", converter = DropdownPropExtConverter.class)
//	@ValidationPropertyExt(dropdownValues = { "0", "1" }, dropdownNames = { "否", "是" }, cellType = ValidationCellEnum.LIST)
//	private String autoPickUpSign;
//
//	@ExcelProperty(value = "提货图片(是/否)", converter = DropdownPropExtConverter.class)
//	@ValidationPropertyExt(dropdownValues = { "0", "1" }, dropdownNames = { "否", "是" }, cellType = ValidationCellEnum.LIST)
//	private String pickUpPicture;
//
//	@ExcelProperty("默认提货围栏(km)")
//	private BigDecimal defaultPickUpRail;
//
//	@ExcelProperty(value = "送达签到(是/否)", converter = DropdownPropExtConverter.class)
//	@ValidationPropertyExt(dropdownValues = { "0", "1" }, dropdownNames = { "否", "是" }, cellType = ValidationCellEnum.LIST)
//	private String deliverySign;
//
//	@ExcelProperty(value = "自动送达签到(是/否)", converter = DropdownPropExtConverter.class)
//	@ValidationPropertyExt(dropdownValues = { "0", "1" }, dropdownNames = { "否", "是" }, cellType = ValidationCellEnum.LIST)
//	private String autoDeliverySign;
//
//	@ExcelProperty(value = "送达图片(是/否)", converter = DropdownPropExtConverter.class)
//	@ValidationPropertyExt(dropdownValues = { "0", "1" }, dropdownNames = { "否", "是" }, cellType = ValidationCellEnum.LIST)
//	private String deliveryPicture;
//
//	@ExcelProperty("默认送达围栏(km)")
//	private BigDecimal defaultDeliveryRail;
//
//	@ExcelProperty(value = "是否要求回单(是/否)", converter = DropdownPropExtConverter.class)
//	@ValidationPropertyExt(dropdownValues = { "0", "1" }, dropdownNames = { "否", "是" }, cellType = ValidationCellEnum.LIST)
//	private String returnOrderFlag;
//
//	@ExcelProperty("回单期限(天)")
//	private Integer returnOrderDeadline;

}
